# (c) 2018, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

''' unit test template for ONTAP Ansible module '''

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch, Mock
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_broadcast_domain \
    import NetAppOntapBroadcastDomain as broadcast_domain_module  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None, data=None):
        ''' save arguments '''
        self.type = kind
        self.params = data
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        if self.type == 'broadcast_domain':
            xml = self.build_broadcast_domain_info(self.params)
        self.xml_out = xml
        return xml

    @staticmethod
    def build_broadcast_domain_info(broadcast_domain_details):
        ''' build xml data for broadcast_domain info '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = {
            'num-records': 1,
            'attributes-list': {
                'net-port-broadcast-domain-info': {
                    'broadcast-domain': broadcast_domain_details['name'],
                    'ipspace': broadcast_domain_details['ipspace'],
                    'mtu': broadcast_domain_details['mtu'],
                    'ports': {
                        'port-info': {
                            'port': 'test_port_1'
                        }
                    }
                }

            }
        }
        xml.translate_struct(attributes)
        return xml


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        self.server = MockONTAPConnection()
        self.mock_broadcast_domain = {
            'name': 'test_broadcast_domain',
            'mtu': '1000',
            'ipspace': 'Default',
            'ports': 'test_port_1'
        }

    def mock_args(self):
        return {
            'name': self.mock_broadcast_domain['name'],
            'ipspace': self.mock_broadcast_domain['ipspace'],
            'mtu': self.mock_broadcast_domain['mtu'],
            'ports': self.mock_broadcast_domain['ports'],
            'hostname': 'test',
            'username': 'test_user',
            'password': 'test_pass!'
        }

    def get_broadcast_domain_mock_object(self, kind=None, data=None):
        """
        Helper method to return an na_ontap_volume object
        :param kind: passes this param to MockONTAPConnection()
        :param data: passes this param to MockONTAPConnection()
        :return: na_ontap_volume object
        """
        broadcast_domain_obj = broadcast_domain_module()
        broadcast_domain_obj.asup_log_for_cserver = Mock(return_value=None)
        broadcast_domain_obj.cluster = Mock()
        broadcast_domain_obj.cluster.invoke_successfully = Mock()
        if kind is None:
            broadcast_domain_obj.server = MockONTAPConnection()
        else:
            if data is None:
                broadcast_domain_obj.server = MockONTAPConnection(kind='broadcast_domain', data=self.mock_broadcast_domain)
            else:
                broadcast_domain_obj.server = MockONTAPConnection(kind='broadcast_domain', data=data)
        return broadcast_domain_obj

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            broadcast_domain_module()
        print('Info: %s' % exc.value.args[0]['msg'])

    def test_get_nonexistent_net_route(self):
        ''' Test if get_broadcast_domain returns None for non-existent broadcast_domain '''
        set_module_args(self.mock_args())
        result = self.get_broadcast_domain_mock_object().get_broadcast_domain()
        assert result is None

    def test_create_error_missing_broadcast_domain(self):
        ''' Test if create throws an error if broadcast_domain is not specified'''
        data = self.mock_args()
        del data['name']
        set_module_args(data)
        with pytest.raises(AnsibleFailJson) as exc:
            self.get_broadcast_domain_mock_object('broadcast_domain').create_broadcast_domain()
        msg = 'missing required arguments: name'
        assert exc.value.args[0]['msg'] == msg

    @patch('ansible.modules.storage.netapp.na_ontap_broadcast_domain.NetAppOntapBroadcastDomain.create_broadcast_domain')
    def test_successful_create(self, create_broadcast_domain):
        ''' Test successful create '''
        data = self.mock_args()
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_broadcast_domain_mock_object().apply()
        assert exc.value.args[0]['changed']
        create_broadcast_domain.assert_called_with()

    def test_create_idempotency(self):
        ''' Test create idempotency '''
        set_module_args(self.mock_args())
        obj = self.get_broadcast_domain_mock_object('broadcast_domain')
        with pytest.raises(AnsibleExitJson) as exc:
            obj.apply()
        assert not exc.value.args[0]['changed']

    def test_modify_mtu(self):
        ''' Test successful modify mtu '''
        data = self.mock_args()
        data['mtu'] = '1200'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_broadcast_domain_mock_object('broadcast_domain').apply()
        assert exc.value.args[0]['changed']

    def test_modify_ipspace_idempotency(self):
        ''' Test modify ipsapce idempotency'''
        data = self.mock_args()
        data['ipspace'] = 'Cluster'
        set_module_args(data)
        with pytest.raises(AnsibleFailJson) as exc:
            self.get_broadcast_domain_mock_object('broadcast_domain').apply()
        msg = 'A domain ipspace can not be modified after the domain has been created.'
        assert exc.value.args[0]['msg'] == msg

    @patch('ansible.modules.storage.netapp.na_ontap_broadcast_domain.NetAppOntapBroadcastDomain.add_broadcast_domain_ports')
    def test_add_ports(self, add_broadcast_domain_ports):
        ''' Test successful modify ports '''
        data = self.mock_args()
        data['ports'] = 'test_port_1,test_port_2'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_broadcast_domain_mock_object('broadcast_domain').apply()
        assert exc.value.args[0]['changed']
        add_broadcast_domain_ports.assert_called_with(['test_port_2'])

    @patch('ansible.modules.storage.netapp.na_ontap_broadcast_domain.NetAppOntapBroadcastDomain.delete_broadcast_domain_ports')
    def test_delete_ports(self, delete_broadcast_domain_ports):
        ''' Test successful modify ports '''
        data = self.mock_args()
        data['ports'] = ''
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_broadcast_domain_mock_object('broadcast_domain').apply()
        assert exc.value.args[0]['changed']
        delete_broadcast_domain_ports.assert_called_with(['test_port_1'])

    @patch('ansible.modules.storage.netapp.na_ontap_broadcast_domain.NetAppOntapBroadcastDomain.modify_broadcast_domain')
    @patch('ansible.modules.storage.netapp.na_ontap_broadcast_domain.NetAppOntapBroadcastDomain.split_broadcast_domain')
    @patch('ansible.modules.storage.netapp.na_ontap_broadcast_domain.NetAppOntapBroadcastDomain.get_broadcast_domain')
    def test_split_broadcast_domain(self, get_broadcast_domain, split_broadcast_domain, modify_broadcast_domain):
        ''' Test successful split broadcast domain '''
        data = self.mock_args()
        data['from_name'] = 'test_broadcast_domain'
        data['name'] = 'test_broadcast_domain_2'
        data['ports'] = 'test_port_2'
        set_module_args(data)
        current = {
            'name': 'test_broadcast_domain',
            'mtu': '1000',
            'ipspace': 'Default',
            'ports': ['test_port_1,test_port2']
        }
        get_broadcast_domain.side_effect = [
            None,
            current,
            current
        ]
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_broadcast_domain_mock_object().apply()
        assert exc.value.args[0]['changed']
        modify_broadcast_domain.assert_not_called()
        split_broadcast_domain.assert_called_with()

    @patch('ansible.modules.storage.netapp.na_ontap_broadcast_domain.NetAppOntapBroadcastDomain.delete_broadcast_domain')
    @patch('ansible.modules.storage.netapp.na_ontap_broadcast_domain.NetAppOntapBroadcastDomain.modify_broadcast_domain')
    @patch('ansible.modules.storage.netapp.na_ontap_broadcast_domain.NetAppOntapBroadcastDomain.get_broadcast_domain')
    def test_split_broadcast_domain_modify_delete(self, get_broadcast_domain, modify_broadcast_domain, delete_broadcast_domain):
        ''' Test successful split broadcast domain '''
        data = self.mock_args()
        data['from_name'] = 'test_broadcast_domain'
        data['name'] = 'test_broadcast_domain_2'
        data['ports'] = 'test_port_1,test_port_2'
        data['mtu'] = '1200'
        set_module_args(data)

        current = {
            'name': 'test_broadcast_domain',
            'mtu': '1000',
            'ipspace': 'Default',
            'ports': ['test_port_1,test_port2']
        }
        get_broadcast_domain.side_effect = [
            None,
            current,
            current
        ]
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_broadcast_domain_mock_object().apply()
        assert exc.value.args[0]['changed']
        delete_broadcast_domain.assert_called_with('test_broadcast_domain')
        modify_broadcast_domain.assert_called_with()

    @patch('ansible.modules.storage.netapp.na_ontap_broadcast_domain.NetAppOntapBroadcastDomain.get_broadcast_domain')
    def test_split_broadcast_domain_not_exist(self, get_broadcast_domain):
        ''' Test successful split broadcast domain '''
        data = self.mock_args()
        data['from_name'] = 'test_broadcast_domain'
        data['name'] = 'test_broadcast_domain_2'
        data['ports'] = 'test_port_2'
        set_module_args(data)

        get_broadcast_domain.side_effect = [
            None,
            None,
        ]
        with pytest.raises(AnsibleFailJson) as exc:
            self.get_broadcast_domain_mock_object().apply()
        msg = 'A domain can not be split if it does not exist.'
        assert exc.value.args[0]['msg'], msg

    @patch('ansible.modules.storage.netapp.na_ontap_broadcast_domain.NetAppOntapBroadcastDomain.split_broadcast_domain')
    def test_split_broadcast_domain_idempotency(self, split_broadcast_domain):
        ''' Test successful split broadcast domain '''
        data = self.mock_args()
        data['from_name'] = 'test_broadcast_domain'
        data['ports'] = 'test_port_1'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_broadcast_domain_mock_object('broadcast_domain').apply()
        assert exc.value.args[0]['changed'] is False
        split_broadcast_domain.assert_not_called()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2017 F5 Networks Inc.
# GNU General Public License v3.0 (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import json
import pytest
import sys

if sys.version_info < (2, 7):
    pytestmark = pytest.mark.skip("F5 Ansible modules require Python >= 2.7")

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.six import iteritems

try:
    from library.modules.bigip_device_facts import Parameters
    from library.modules.bigip_device_facts import VirtualAddressesFactManager
    from library.modules.bigip_device_facts import VirtualAddressesParameters
    from library.modules.bigip_device_facts import ArgumentSpec
    from library.modules.bigip_device_facts import ModuleManager

    # In Ansible 2.8, Ansible changed import paths.
    from test.units.compat import unittest
    from test.units.compat.mock import Mock
    from test.units.compat.mock import patch

    from test.units.modules.utils import set_module_args
except ImportError:
    from ansible.modules.network.f5.bigip_device_facts import Parameters
    from ansible.modules.network.f5.bigip_device_facts import VirtualAddressesFactManager
    from ansible.modules.network.f5.bigip_device_facts import VirtualAddressesParameters
    from ansible.modules.network.f5.bigip_device_facts import ArgumentSpec
    from ansible.modules.network.f5.bigip_device_facts import ModuleManager

    # Ansible 2.8 imports
    from units.compat import unittest
    from units.compat.mock import Mock
    from units.compat.mock import patch

    from units.modules.utils import set_module_args


fixture_path = os.path.join(os.path.dirname(__file__), 'fixtures')
fixture_data = {}


def load_fixture(name):
    path = os.path.join(fixture_path, name)

    if path in fixture_data:
        return fixture_data[path]

    with open(path) as f:
        data = f.read()

    try:
        data = json.loads(data)
    except Exception:
        pass

    fixture_data[path] = data
    return data


class FakeVirtualAddress:
    def __init__(self, *args, **kwargs):
        attrs = kwargs.pop('params', {})
        for key, value in iteritems(attrs):
            setattr(self, key, value)


class TestParameters(unittest.TestCase):
    def test_module_parameters(self):
        args = dict(
            gather_subset=['virtual-servers'],
        )
        p = Parameters(params=args)
        assert p.gather_subset == ['virtual-servers']


class TestManager(unittest.TestCase):

    def setUp(self):
        self.spec = ArgumentSpec()

        try:
            self.p1 = patch('library.modules.bigip_device_facts.modules_provisioned')
            self.m1 = self.p1.start()
            self.m1.return_value = ['ltm', 'gtm', 'asm']
        except Exception:
            self.p1 = patch('ansible.modules.network.f5.bigip_device_facts.modules_provisioned')
            self.m1 = self.p1.start()
            self.m1.return_value = ['ltm', 'gtm', 'asm']

    def tearDown(self):
        self.p1.stop()

    def test_get_trunk_facts(self, *args):
        set_module_args(dict(
            gather_subset=['virtual-addresses'],
            provider=dict(
                server='localhost',
                password='password',
                user='admin'
            )
        ))

        fixture1 = load_fixture('load_ltm_virtual_address_collection_1.json')
        collection = fixture1['items']

        module = AnsibleModule(
            argument_spec=self.spec.argument_spec,
            supports_check_mode=self.spec.supports_check_mode
        )

        tm = VirtualAddressesFactManager(module=module)
        tm.read_collection_from_device = Mock(return_value=collection)

        # Override methods to force specific logic in the module to happen
        mm = ModuleManager(module=module)
        mm.get_manager = Mock(return_value=tm)

        results = mm.exec_module()

        assert results['changed'] is True
        assert 'virtual_addresses' in results
        assert len(results['virtual_addresses']) > 0



#
# (c) 2019, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from ansible.module_utils.network.eos.providers.cli.config.bgp.process import Provider
from ansible.modules.network.eos import eos_bgp
from .eos_module import TestEosModule, load_fixture


class TestFrrBgpModule(TestEosModule):
    module = eos_bgp

    def setUp(self):
        super(TestFrrBgpModule, self).setUp()
        self._bgp_config = load_fixture('eos_bgp_config.cfg')

    def test_eos_bgp(self):
        obj = Provider(params=dict(config=dict(bgp_as=64496, router_id='192.0.2.2', networks=None,
                                               address_family=None), operation='merge'))
        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, ['router bgp 64496', 'router-id 192.0.2.2', 'exit'])

    def test_eos_bgp_idempotent(self):
        obj = Provider(params=dict(config=dict(bgp_as=64496, router_id='192.0.2.1',
                                               networks=None, address_family=None), operation='merge'))
        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, [])

    def test_eos_bgp_remove(self):
        obj = Provider(params=dict(config=dict(bgp_as=64496, networks=None, address_family=None), operation='delete'))
        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, ['no router bgp 64496'])

    def test_eos_bgp_neighbor(self):
        obj = Provider(params=dict(config=dict(bgp_as=64496, neighbors=[dict(neighbor='198.51.100.12', remote_as=64498)],
                                               networks=None, address_family=None),
                                   operation='merge'))
        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, ['router bgp 64496', 'neighbor 198.51.100.12 remote-as 64498', 'exit'])

    def test_eos_bgp_neighbor_idempotent(self):
        neighbors = [dict(neighbor='198.51.100.102', remote_as=64498, timers=dict(keepalive=300, holdtime=360)),
                     dict(neighbor='203.0.113.5', remote_as=64511, maximum_prefix=500)]
        obj = Provider(params=dict(config=dict(bgp_as=64496, neighbors=neighbors, networks=None, address_family=None),
                                   operation='merge'))
        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, [])

    def test_eos_bgp_network(self):
        obj = Provider(
            params=dict(config=dict(bgp_as=64496, networks=[dict(prefix='203.0.113.0', masklen=24, route_map='RMAP_1')],
                                    address_family=None),
                        operation='merge'))
        commands = obj.render(self._bgp_config)
        self.assertEqual(sorted(commands), sorted(['router bgp 64496', 'network 203.0.113.0/24 route-map RMAP_1', 'exit']))

    def test_eos_bgp_network_idempotent(self):
        obj = Provider(
            params=dict(config=dict(bgp_as=64496, networks=[dict(prefix='192.0.2.0', masklen=27, route_map='RMAP_1'),
                                                            dict(prefix='198.51.100.0', masklen=24, route_map='RMAP_2')],
                                    address_family=None),
                        operation='merge'))
        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, [])

    def test_eos_bgp_redistribute(self):
        rd_1 = dict(protocol='rip', route_map='RMAP_1')

        config = dict(bgp_as=64496, redistribute=[rd_1], networks=None, address_family=None)

        obj = Provider(params=dict(config=config, operation='merge'))

        commands = obj.render(self._bgp_config)
        cmd = ['router bgp 64496', 'redistribute rip route-map RMAP_1', 'exit']
        self.assertEqual(sorted(commands), sorted(cmd))

    def test_eos_bgp_redistribute_idempotent(self):
        rd_1 = dict(protocol='ospf', route_map='RMAP_1')
        config = dict(bgp_as=64496, redistribute=[rd_1], networks=None, address_family=None)

        obj = Provider(params=dict(config=config, operation='merge'))

        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, [])

    def test_eos_bgp_address_family_neighbors(self):
        af_nbr_1 = dict(neighbor='198.51.100.104', default_originate=True, activate=True)
        af_nbr_2 = dict(neighbor='198.51.100.105', activate=True, weight=30, graceful_restart=True)

        config = dict(bgp_as=64496, address_family=[dict(afi='ipv4', neighbors=[af_nbr_1, af_nbr_2])],
                      networks=None)

        obj = Provider(params=dict(config=config, operation='merge'))

        commands = obj.render(self._bgp_config)
        cmd = ['router bgp 64496', 'address-family ipv4', 'neighbor 198.51.100.104 activate',
               'neighbor 198.51.100.104 default-originate', 'neighbor 198.51.100.105 weight 30',
               'neighbor 198.51.100.105 activate', 'neighbor 198.51.100.105 graceful-restart', 'exit', 'exit']
        self.assertEqual(sorted(commands), sorted(cmd))

    def test_eos_bgp_address_family_neighbors_idempotent(self):
        af_nbr_1 = dict(neighbor='198.51.100.102', activate=True, graceful_restart=True, default_originate=True, weight=25)
        af_nbr_2 = dict(neighbor='192.0.2.111', activate=True, default_originate=True)
        config = dict(bgp_as=64496, address_family=[dict(afi='ipv4', neighbors=[af_nbr_1, af_nbr_2])],
                      networks=None)

        obj = Provider(params=dict(config=config, operation='merge'))

        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, [])

    def test_eos_bgp_address_family_networks(self):
        net = dict(prefix='203.0.113.128', masklen=26, route_map='RMAP_1')
        net2 = dict(prefix='203.0.113.192', masklen=26, route_map='RMAP_2')

        config = dict(bgp_as=64496, address_family=[dict(afi='ipv4', networks=[net, net2])],
                      networks=None)

        obj = Provider(params=dict(config=config, operation='merge'))

        commands = obj.render(self._bgp_config)
        cmd = ['router bgp 64496', 'address-family ipv4', 'network 203.0.113.128/26 route-map RMAP_1',
               'network 203.0.113.192/26 route-map RMAP_2', 'exit', 'exit']
        self.assertEqual(sorted(commands), sorted(cmd))

    def test_eos_bgp_address_family_networks_idempotent(self):
        net = dict(prefix='2001:db8:8000::', masklen=34, route_map=None)
        net2 = dict(prefix='2001:db8:c000::', masklen=34, route_map=None)

        config = dict(bgp_as=64496, address_family=[dict(afi='ipv6', networks=[net, net2])],
                      networks=None)

        obj = Provider(params=dict(config=config, operation='merge'))

        commands = obj.render(self._bgp_config)
        self.assertEqual(commands, [])

    def test_eos_bgp_operation_override(self):
        net_1 = dict(prefix='2001:0db8:0800::', masklen=38, route_map='RMAP_1')
        net_2 = dict(prefix='2001:0db8:1c00::', masklen=38, route_map='RMAP_2')
        nbr_1 = dict(neighbor='203.0.113.111', remote_as=64511, update_source='Ethernet2')
        nbr_2 = dict(neighbor='203.0.113.120', remote_as=64511, timers=dict(keepalive=300, holdtime=360))
        af_nbr_1 = dict(neighbor='203.0.113.111', activate=True)
        af_nbr_2 = dict(neighbor='203.0.113.120', activate=True, default_originate=True)

        af_1 = dict(afi='ipv4', neighbors=[af_nbr_1, af_nbr_2])
        af_2 = dict(afi='ipv6', networks=[net_1, net_2])
        config = dict(bgp_as=64496, neighbors=[nbr_1, nbr_2], address_family=[af_1, af_2],
                      networks=None)

        obj = Provider(params=dict(config=config, operation='override'))
        commands = obj.render(self._bgp_config)

        cmd = ['no router bgp 64496', 'router bgp 64496', 'neighbor 203.0.113.111 remote-as 64511',
               'neighbor 203.0.113.111 update-source Ethernet2', 'neighbor 203.0.113.120 remote-as 64511',
               'neighbor 203.0.113.120 timers 300 360', 'address-family ipv4',
               'neighbor 203.0.113.111 activate', 'neighbor 203.0.113.120 default-originate', 'neighbor 203.0.113.120 activate',
               'exit', 'address-family ipv6', 'network 2001:0db8:0800::/38 route-map RMAP_1',
               'network 2001:0db8:1c00::/38 route-map RMAP_2',
               'exit', 'exit']

        self.assertEqual(sorted(commands), sorted(cmd))

    def test_eos_bgp_operation_replace(self):
        net = dict(prefix='203.0.113.0', masklen=27, route_map='RMAP_1')
        net2 = dict(prefix='192.0.2.32', masklen=29, route_map='RMAP_2')
        net_3 = dict(prefix='2001:db8:8000::', masklen=34, route_map=None)
        net_4 = dict(prefix='2001:db8:c000::', masklen=34, route_map=None)

        af_1 = dict(afi='ipv4', networks=[net, net2])
        af_2 = dict(afi='ipv6', networks=[net_3, net_4])

        config = dict(bgp_as=64496, address_family=[af_1, af_2], networks=None)
        obj = Provider(params=dict(config=config, operation='replace'))
        commands = obj.render(self._bgp_config)

        cmd = ['router bgp 64496', 'address-family ipv4', 'network 203.0.113.0/27 route-map RMAP_1',
               'network 192.0.2.32/29 route-map RMAP_2', 'no network 192.0.2.0/27', 'no network 198.51.100.0/24',
               'exit', 'exit']

        self.assertEqual(sorted(commands), sorted(cmd))

    def test_eos_bgp_operation_replace_with_new_as(self):
        nbr = dict(neighbor='203.0.113.124', remote_as=64496, update_source='Ethernet3')

        config = dict(bgp_as=64497, neighbors=[nbr], networks=None, address_family=None)
        obj = Provider(params=dict(config=config, operation='replace'))
        commands = obj.render(self._bgp_config)

        cmd = ['no router bgp 64496', 'router bgp 64497', 'neighbor 203.0.113.124 remote-as 64496',
               'neighbor 203.0.113.124 update-source Ethernet3', 'exit']

        self.assertEqual(sorted(commands), sorted(cmd))



# (c) 2018, NetApp, Inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

''' unit test template for ONTAP Ansible module '''

from __future__ import print_function
import json
import pytest

from units.compat import unittest
from units.compat.mock import patch, Mock
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_svm \
    import NetAppOntapSVM as svm_module  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None, data=None):
        ''' save arguments '''
        self.type = kind
        self.params = data
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        if self.type == 'vserver':
            xml = self.build_vserver_info(self.params)
        self.xml_out = xml
        return xml

    @staticmethod
    def build_vserver_info(vserver):
        ''' build xml data for vserser-info '''
        xml = netapp_utils.zapi.NaElement('xml')
        data = {'num-records': 1, 'attributes-list': {'vserver-info': {
            'vserver-name': vserver['name'],
            'ipspace': vserver['ipspace'],
            'root-volume': vserver['root_volume'],
            'root-volume-aggregate': vserver['root_volume_aggregate'],
            'language': vserver['language'],
            'comment': vserver['comment'],
            'snapshot-policy': vserver['snapshot_policy'],
            'vserver-subtype': vserver['subtype'],
            'allowed-protocols': [{'protocol': 'nfs'}, {'protocol': 'cifs'}],
            'aggr-list': [{'aggr-name': 'aggr_1'}, {'aggr-name': 'aggr_2'}],
        }}}
        xml.translate_struct(data)
        return xml


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        self.server = MockONTAPConnection()
        self.mock_vserver = {
            'name': 'test_svm',
            'root_volume': 'ansible_vol',
            'root_volume_aggregate': 'ansible_aggr',
            'aggr_list': 'aggr_1,aggr_2',
            'ipspace': 'ansible_ipspace',
            'subtype': 'default',
            'language': 'c.utf_8',
            'snapshot_policy': 'old_snapshot_policy',
            'comment': 'this is a comment'
        }

    def mock_args(self):
        return {
            'name': self.mock_vserver['name'],
            'root_volume': self.mock_vserver['root_volume'],
            'root_volume_aggregate': self.mock_vserver['root_volume_aggregate'],
            'aggr_list': self.mock_vserver['aggr_list'],
            'ipspace': self.mock_vserver['ipspace'],
            'comment': self.mock_vserver['comment'],
            'subtype': 'default',
            'hostname': 'test',
            'username': 'test_user',
            'password': 'test_pass!'
        }

    def get_vserver_mock_object(self, kind=None, data=None):
        """
        Helper method to return an na_ontap_volume object
        :param kind: passes this param to MockONTAPConnection()
        :param data: passes this param to MockONTAPConnection()
        :return: na_ontap_volume object
        """
        vserver_obj = svm_module()
        vserver_obj.asup_log_for_cserver = Mock(return_value=None)
        vserver_obj.cluster = Mock()
        vserver_obj.cluster.invoke_successfully = Mock()
        if kind is None:
            vserver_obj.server = MockONTAPConnection()
        else:
            if data is None:
                vserver_obj.server = MockONTAPConnection(kind='vserver', data=self.mock_vserver)
            else:
                vserver_obj.server = MockONTAPConnection(kind='vserver', data=data)
        return vserver_obj

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            svm_module()
        print('Info: %s' % exc.value.args[0]['msg'])

    def test_get_nonexistent_vserver(self):
        ''' test if get_vserver() throws an error if vserver is not specified '''
        data = self.mock_args()
        set_module_args(data)
        result = self.get_vserver_mock_object().get_vserver()
        assert result is None

    def test_create_error_missing_name(self):
        ''' Test if create throws an error if name is not specified'''
        data = self.mock_args()
        del data['name']
        set_module_args(data)
        with pytest.raises(AnsibleFailJson) as exc:
            self.get_vserver_mock_object('vserver').create_vserver()
        msg = 'missing required arguments: name'
        assert exc.value.args[0]['msg'] == msg

    @patch('ansible.modules.storage.netapp.na_ontap_svm.NetAppOntapSVM.create_vserver')
    def test_successful_create(self, create_vserver):
        '''Test successful create'''
        data = self.mock_args()
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_vserver_mock_object().apply()
        assert exc.value.args[0]['changed']
        create_vserver.assert_called_with()

    @patch('ansible.modules.storage.netapp.na_ontap_svm.NetAppOntapSVM.create_vserver')
    def test_create_idempotency(self, create_vserver):
        '''Test successful create'''
        data = self.mock_args()
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_vserver_mock_object('vserver').apply()
        assert not exc.value.args[0]['changed']
        create_vserver.assert_not_called()

    def test_successful_delete(self):
        '''Test successful delete'''
        data = self.mock_args()
        data['state'] = 'absent'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_vserver_mock_object('vserver').apply()
        assert exc.value.args[0]['changed']

    @patch('ansible.modules.storage.netapp.na_ontap_svm.NetAppOntapSVM.delete_vserver')
    def test_delete_idempotency(self, delete_vserver):
        '''Test delete idempotency'''
        data = self.mock_args()
        data['state'] = 'absent'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_vserver_mock_object().apply()
        assert not exc.value.args[0]['changed']
        delete_vserver.assert_not_called()

    @patch('ansible.modules.storage.netapp.na_ontap_svm.NetAppOntapSVM.get_vserver')
    def test_successful_rename(self, get_vserver):
        '''Test successful rename'''
        data = self.mock_args()
        data['from_name'] = 'test_svm'
        data['name'] = 'test_new_svm'
        set_module_args(data)
        current = {
            'name': 'test_svm',
            'root_volume': 'ansible_vol',
            'root_volume_aggregate': 'ansible_aggr',
            'ipspace': 'ansible_ipspace',
            'subtype': 'default',
            'language': 'c.utf_8'
        }
        get_vserver.side_effect = [
            None,
            current
        ]
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_vserver_mock_object().apply()
        assert exc.value.args[0]['changed']

    def test_successful_modify_language(self):
        '''Test successful modify language'''
        data = self.mock_args()
        data['language'] = 'c'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_vserver_mock_object('vserver').apply()
        assert exc.value.args[0]['changed']

    def test_successful_modify_snapshot_policy(self):
        '''Test successful modify language'''
        data = self.mock_args()
        data['snapshot_policy'] = 'new_snapshot_policy'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_vserver_mock_object('vserver').apply()
        assert exc.value.args[0]['changed']

    def test_successful_modify_allowed_protocols(self):
        '''Test successful modify allowed protocols'''
        data = self.mock_args()
        data['allowed_protocols'] = 'protocol_1,protocol_2'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_vserver_mock_object('vserver').apply()
        assert exc.value.args[0]['changed']

    def test_successful_modify_aggr_list(self):
        '''Test successful modify aggr-list'''
        data = self.mock_args()
        data['aggr_list'] = 'aggr_3,aggr_4'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_vserver_mock_object('vserver').apply()
        assert exc.value.args[0]['changed']



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2018 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_firewall_ippool6
short_description: Configure IPv6 IP pools in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure firewall feature and ippool6 category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    firewall_ippool6:
        description:
            - Configure IPv6 IP pools.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            comments:
                description:
                    - Comment.
            endip:
                description:
                    - "Final IPv6 address (inclusive) in the range for the address pool (format xxxx:xxxx:xxxx:xxxx:xxxx:xxxx:xxxx:xxxx, Default: ::)."
            name:
                description:
                    - IPv6 IP pool name.
                required: true
            startip:
                description:
                    - "First IPv6 address (inclusive) in the range for the address pool (format xxxx:xxxx:xxxx:xxxx:xxxx:xxxx:xxxx:xxxx, Default: ::)."
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure IPv6 IP pools.
    fortios_firewall_ippool6:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      firewall_ippool6:
        state: "present"
        comments: "<your_own_comment>"
        endip: "<your_own_value>"
        name: "default_name_5"
        startip: "<your_own_value>"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "key1"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_firewall_ippool6_data(json):
    option_list = ['comments', 'endip', 'name',
                   'startip']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def firewall_ippool6(data, fos):
    vdom = data['vdom']
    firewall_ippool6_data = data['firewall_ippool6']
    filtered_data = filter_firewall_ippool6_data(firewall_ippool6_data)
    if firewall_ippool6_data['state'] == "present":
        return fos.set('firewall',
                       'ippool6',
                       data=filtered_data,
                       vdom=vdom)

    elif firewall_ippool6_data['state'] == "absent":
        return fos.delete('firewall',
                          'ippool6',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_firewall(data, fos):
    login(data)

    methodlist = ['firewall_ippool6']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "firewall_ippool6": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "comments": {"required": False, "type": "str"},
                "endip": {"required": False, "type": "str"},
                "name": {"required": True, "type": "str"},
                "startip": {"required": False, "type": "str"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_firewall(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 Google
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# ----------------------------------------------------------------------------
#
#     ***     AUTO GENERATED CODE    ***    AUTO GENERATED CODE     ***
#
# ----------------------------------------------------------------------------
#
#     This file is automatically generated by Magic Modules and manual
#     changes will be clobbered when the file is regenerated.
#
#     Please read more about how to change this file at
#     https://www.github.com/GoogleCloudPlatform/magic-modules
#
# ----------------------------------------------------------------------------

from __future__ import absolute_import, division, print_function

__metaclass__ = type

################################################################################
# Documentation
################################################################################

ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ["preview"], 'supported_by': 'community'}

DOCUMENTATION = '''
---
module: gcp_iam_role_facts
description:
- Gather facts for GCP Role
short_description: Gather facts for GCP Role
version_added: 2.8
author: Google Inc. (@googlecloudplatform)
requirements:
- python >= 2.6
- requests >= 2.18.4
- google-auth >= 1.3.0
options: {}
extends_documentation_fragment: gcp
'''

EXAMPLES = '''
- name: " a role facts"
  gcp_iam_role_facts:
    project: test_project
    auth_kind: serviceaccount
    service_account_file: "/tmp/auth.pem"
    state: facts
'''

RETURN = '''
items:
  description: List of items
  returned: always
  type: complex
  contains:
    name:
      description:
      - The name of the role.
      returned: success
      type: str
    title:
      description:
      - A human-readable title for the role. Typically this is limited to 100 UTF-8
        bytes.
      returned: success
      type: str
    description:
      description:
      - Human-readable description for the role.
      returned: success
      type: str
    includedPermissions:
      description:
      - Names of permissions this role grants when bound in an IAM policy.
      returned: success
      type: list
    stage:
      description:
      - The current launch stage of the role.
      returned: success
      type: str
    deleted:
      description:
      - The current deleted state of the role.
      returned: success
      type: bool
'''

################################################################################
# Imports
################################################################################
from ansible.module_utils.gcp_utils import navigate_hash, GcpSession, GcpModule, GcpRequest
import json

################################################################################
# Main
################################################################################


def main():
    module = GcpModule(argument_spec=dict())

    if not module.params['scopes']:
        module.params['scopes'] = ['https://www.googleapis.com/auth/iam']

    items = fetch_list(module, collection(module))
    if items.get('roles'):
        items = items.get('roles')
    else:
        items = []
    return_value = {'items': items}
    module.exit_json(**return_value)


def collection(module):
    return "https://iam.googleapis.com/v1/projects/{project}/roles".format(**module.params)


def fetch_list(module, link):
    auth = GcpSession(module, 'iam')
    response = auth.get(link)
    return return_if_object(module, response)


def return_if_object(module, response):
    # If not found, return nothing.
    if response.status_code == 404:
        return None

    # If no content, return nothing.
    if response.status_code == 204:
        return None

    try:
        module.raise_for_status(response)
        result = response.json()
    except getattr(json.decoder, 'JSONDecodeError', ValueError) as inst:
        module.fail_json(msg="Invalid JSON response with error: %s" % inst)

    if navigate_hash(result, ['error', 'errors']):
        module.fail_json(msg=navigate_hash(result, ['error', 'errors']))

    return result


if __name__ == "__main__":
    main()



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_user_radius
short_description: Configure RADIUS server entries in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify user feature and radius category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    user_radius:
        description:
            - Configure RADIUS server entries.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            accounting-server:
                description:
                    - Additional accounting servers.
                suboptions:
                    id:
                        description:
                            - ID (0 - 4294967295).
                        required: true
                    port:
                        description:
                            - RADIUS accounting port number.
                    secret:
                        description:
                            - Secret key.
                    server:
                        description:
                            - Server CN domain name or IP.
                    source-ip:
                        description:
                            - Source IP address for communications to the RADIUS server.
                    status:
                        description:
                            - Status.
                        choices:
                            - enable
                            - disable
            acct-all-servers:
                description:
                    - Enable/disable sending of accounting messages to all configured servers (default = disable).
                choices:
                    - enable
                    - disable
            acct-interim-interval:
                description:
                    - Time in seconds between each accounting interim update message.
            all-usergroup:
                description:
                    - Enable/disable automatically including this RADIUS server in all user groups.
                choices:
                    - disable
                    - enable
            auth-type:
                description:
                    - Authentication methods/protocols permitted for this RADIUS server.
                choices:
                    - auto
                    - ms_chap_v2
                    - ms_chap
                    - chap
                    - pap
            class:
                description:
                    - Class attribute name(s).
                suboptions:
                    name:
                        description:
                            - Class name.
                        required: true
            h3c-compatibility:
                description:
                    - Enable/disable compatibility with the H3C, a mechanism that performs security checking for authentication.
                choices:
                    - enable
                    - disable
            name:
                description:
                    - RADIUS server entry name.
                required: true
            nas-ip:
                description:
                    - IP address used to communicate with the RADIUS server and used as NAS-IP-Address and Called-Station-ID attributes.
            password-encoding:
                description:
                    - Password encoding.
                choices:
                    - auto
                    - ISO-8859-1
            password-renewal:
                description:
                    - Enable/disable password renewal.
                choices:
                    - enable
                    - disable
            radius-coa:
                description:
                    - Enable to allow a mechanism to change the attributes of an authentication, authorization, and accounting session after it is
                       authenticated.
                choices:
                    - enable
                    - disable
            radius-port:
                description:
                    - RADIUS service port number.
            rsso:
                description:
                    - Enable/disable RADIUS based single sign on feature.
                choices:
                    - enable
                    - disable
            rsso-context-timeout:
                description:
                    - Time in seconds before the logged out user is removed from the "user context list" of logged on users.
            rsso-endpoint-attribute:
                description:
                    - RADIUS attributes used to extract the user end point identifier from the RADIUS Start record.
                choices:
                    - User-Name
                    - NAS-IP-Address
                    - Framed-IP-Address
                    - Framed-IP-Netmask
                    - Filter-Id
                    - Login-IP-Host
                    - Reply-Message
                    - Callback-Number
                    - Callback-Id
                    - Framed-Route
                    - Framed-IPX-Network
                    - Class
                    - Called-Station-Id
                    - Calling-Station-Id
                    - NAS-Identifier
                    - Proxy-State
                    - Login-LAT-Service
                    - Login-LAT-Node
                    - Login-LAT-Group
                    - Framed-AppleTalk-Zone
                    - Acct-Session-Id
                    - Acct-Multi-Session-Id
            rsso-endpoint-block-attribute:
                description:
                    - RADIUS attributes used to block a user.
                choices:
                    - User-Name
                    - NAS-IP-Address
                    - Framed-IP-Address
                    - Framed-IP-Netmask
                    - Filter-Id
                    - Login-IP-Host
                    - Reply-Message
                    - Callback-Number
                    - Callback-Id
                    - Framed-Route
                    - Framed-IPX-Network
                    - Class
                    - Called-Station-Id
                    - Calling-Station-Id
                    - NAS-Identifier
                    - Proxy-State
                    - Login-LAT-Service
                    - Login-LAT-Node
                    - Login-LAT-Group
                    - Framed-AppleTalk-Zone
                    - Acct-Session-Id
                    - Acct-Multi-Session-Id
            rsso-ep-one-ip-only:
                description:
                    - Enable/disable the replacement of old IP addresses with new ones for the same endpoint on RADIUS accounting Start messages.
                choices:
                    - enable
                    - disable
            rsso-flush-ip-session:
                description:
                    - Enable/disable flushing user IP sessions on RADIUS accounting Stop messages.
                choices:
                    - enable
                    - disable
            rsso-log-flags:
                description:
                    - Events to log.
                choices:
                    - protocol-error
                    - profile-missing
                    - accounting-stop-missed
                    - accounting-event
                    - endpoint-block
                    - radiusd-other
                    - none
            rsso-log-period:
                description:
                    - Time interval in seconds that group event log messages will be generated for dynamic profile events.
            rsso-radius-response:
                description:
                    - Enable/disable sending RADIUS response packets after receiving Start and Stop records.
                choices:
                    - enable
                    - disable
            rsso-radius-server-port:
                description:
                    - UDP port to listen on for RADIUS Start and Stop records.
            rsso-secret:
                description:
                    - RADIUS secret used by the RADIUS accounting server.
            rsso-validate-request-secret:
                description:
                    - Enable/disable validating the RADIUS request shared secret in the Start or End record.
                choices:
                    - enable
                    - disable
            secondary-secret:
                description:
                    - Secret key to access the secondary server.
            secondary-server:
                description:
                    - Secondary RADIUS CN domain name or IP.
            secret:
                description:
                    - Pre-shared secret key used to access the primary RADIUS server.
            server:
                description:
                    - Primary RADIUS server CN domain name or IP address.
            source-ip:
                description:
                    - Source IP address for communications to the RADIUS server.
            sso-attribute:
                description:
                    - RADIUS attribute that contains the profile group name to be extracted from the RADIUS Start record.
                choices:
                    - User-Name
                    - NAS-IP-Address
                    - Framed-IP-Address
                    - Framed-IP-Netmask
                    - Filter-Id
                    - Login-IP-Host
                    - Reply-Message
                    - Callback-Number
                    - Callback-Id
                    - Framed-Route
                    - Framed-IPX-Network
                    - Class
                    - Called-Station-Id
                    - Calling-Station-Id
                    - NAS-Identifier
                    - Proxy-State
                    - Login-LAT-Service
                    - Login-LAT-Node
                    - Login-LAT-Group
                    - Framed-AppleTalk-Zone
                    - Acct-Session-Id
                    - Acct-Multi-Session-Id
            sso-attribute-key:
                description:
                    - Key prefix for SSO group value in the SSO attribute.
            sso-attribute-value-override:
                description:
                    - Enable/disable override old attribute value with new value for the same endpoint.
                choices:
                    - enable
                    - disable
            tertiary-secret:
                description:
                    - Secret key to access the tertiary server.
            tertiary-server:
                description:
                    - Tertiary RADIUS CN domain name or IP.
            timeout:
                description:
                    - Time in seconds between re-sending authentication requests.
            use-management-vdom:
                description:
                    - Enable/disable using management VDOM to send requests.
                choices:
                    - enable
                    - disable
            username-case-sensitive:
                description:
                    - Enable/disable case sensitive user names.
                choices:
                    - enable
                    - disable
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure RADIUS server entries.
    fortios_user_radius:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      user_radius:
        state: "present"
        accounting-server:
         -
            id:  "4"
            port: "5"
            secret: "<your_own_value>"
            server: "192.168.100.40"
            source-ip: "84.230.14.43"
            status: "enable"
        acct-all-servers: "enable"
        acct-interim-interval: "11"
        all-usergroup: "disable"
        auth-type: "auto"
        class:
         -
            name: "default_name_15"
        h3c-compatibility: "enable"
        name: "default_name_17"
        nas-ip: "<your_own_value>"
        password-encoding: "auto"
        password-renewal: "enable"
        radius-coa: "enable"
        radius-port: "22"
        rsso: "enable"
        rsso-context-timeout: "24"
        rsso-endpoint-attribute: "User-Name"
        rsso-endpoint-block-attribute: "User-Name"
        rsso-ep-one-ip-only: "enable"
        rsso-flush-ip-session: "enable"
        rsso-log-flags: "protocol-error"
        rsso-log-period: "30"
        rsso-radius-response: "enable"
        rsso-radius-server-port: "32"
        rsso-secret: "<your_own_value>"
        rsso-validate-request-secret: "enable"
        secondary-secret: "<your_own_value>"
        secondary-server: "<your_own_value>"
        secret: "<your_own_value>"
        server: "192.168.100.40"
        source-ip: "84.230.14.43"
        sso-attribute: "User-Name"
        sso-attribute-key: "<your_own_value>"
        sso-attribute-value-override: "enable"
        tertiary-secret: "<your_own_value>"
        tertiary-server: "<your_own_value>"
        timeout: "45"
        use-management-vdom: "enable"
        username-case-sensitive: "enable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_user_radius_data(json):
    option_list = ['accounting-server', 'acct-all-servers', 'acct-interim-interval',
                   'all-usergroup', 'auth-type', 'class',
                   'h3c-compatibility', 'name', 'nas-ip',
                   'password-encoding', 'password-renewal', 'radius-coa',
                   'radius-port', 'rsso', 'rsso-context-timeout',
                   'rsso-endpoint-attribute', 'rsso-endpoint-block-attribute', 'rsso-ep-one-ip-only',
                   'rsso-flush-ip-session', 'rsso-log-flags', 'rsso-log-period',
                   'rsso-radius-response', 'rsso-radius-server-port', 'rsso-secret',
                   'rsso-validate-request-secret', 'secondary-secret', 'secondary-server',
                   'secret', 'server', 'source-ip',
                   'sso-attribute', 'sso-attribute-key', 'sso-attribute-value-override',
                   'tertiary-secret', 'tertiary-server', 'timeout',
                   'use-management-vdom', 'username-case-sensitive']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def flatten_multilists_attributes(data):
    multilist_attrs = []

    for attr in multilist_attrs:
        try:
            path = "data['" + "']['".join(elem for elem in attr) + "']"
            current_val = eval(path)
            flattened_val = ' '.join(elem for elem in current_val)
            exec(path + '= flattened_val')
        except BaseException:
            pass

    return data


def user_radius(data, fos):
    vdom = data['vdom']
    user_radius_data = data['user_radius']
    flattened_data = flatten_multilists_attributes(user_radius_data)
    filtered_data = filter_user_radius_data(flattened_data)
    if user_radius_data['state'] == "present":
        return fos.set('user',
                       'radius',
                       data=filtered_data,
                       vdom=vdom)

    elif user_radius_data['state'] == "absent":
        return fos.delete('user',
                          'radius',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_user(data, fos):
    login(data)

    if data['user_radius']:
        resp = user_radius(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "user_radius": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "accounting-server": {"required": False, "type": "list",
                                      "options": {
                                          "id": {"required": True, "type": "int"},
                                          "port": {"required": False, "type": "int"},
                                          "secret": {"required": False, "type": "str"},
                                          "server": {"required": False, "type": "str"},
                                          "source-ip": {"required": False, "type": "str"},
                                          "status": {"required": False, "type": "str",
                                                     "choices": ["enable", "disable"]}
                                      }},
                "acct-all-servers": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "acct-interim-interval": {"required": False, "type": "int"},
                "all-usergroup": {"required": False, "type": "str",
                                  "choices": ["disable", "enable"]},
                "auth-type": {"required": False, "type": "str",
                              "choices": ["auto", "ms_chap_v2", "ms_chap",
                                          "chap", "pap"]},
                "class": {"required": False, "type": "list",
                          "options": {
                              "name": {"required": True, "type": "str"}
                          }},
                "h3c-compatibility": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "name": {"required": True, "type": "str"},
                "nas-ip": {"required": False, "type": "str"},
                "password-encoding": {"required": False, "type": "str",
                                      "choices": ["auto", "ISO-8859-1"]},
                "password-renewal": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "radius-coa": {"required": False, "type": "str",
                               "choices": ["enable", "disable"]},
                "radius-port": {"required": False, "type": "int"},
                "rsso": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]},
                "rsso-context-timeout": {"required": False, "type": "int"},
                "rsso-endpoint-attribute": {"required": False, "type": "str",
                                            "choices": ["User-Name", "NAS-IP-Address", "Framed-IP-Address",
                                                        "Framed-IP-Netmask", "Filter-Id", "Login-IP-Host",
                                                        "Reply-Message", "Callback-Number", "Callback-Id",
                                                        "Framed-Route", "Framed-IPX-Network", "Class",
                                                        "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier",
                                                        "Proxy-State", "Login-LAT-Service", "Login-LAT-Node",
                                                        "Login-LAT-Group", "Framed-AppleTalk-Zone", "Acct-Session-Id",
                                                        "Acct-Multi-Session-Id"]},
                "rsso-endpoint-block-attribute": {"required": False, "type": "str",
                                                  "choices": ["User-Name", "NAS-IP-Address", "Framed-IP-Address",
                                                              "Framed-IP-Netmask", "Filter-Id", "Login-IP-Host",
                                                              "Reply-Message", "Callback-Number", "Callback-Id",
                                                              "Framed-Route", "Framed-IPX-Network", "Class",
                                                              "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier",
                                                              "Proxy-State", "Login-LAT-Service", "Login-LAT-Node",
                                                              "Login-LAT-Group", "Framed-AppleTalk-Zone", "Acct-Session-Id",
                                                              "Acct-Multi-Session-Id"]},
                "rsso-ep-one-ip-only": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "rsso-flush-ip-session": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                "rsso-log-flags": {"required": False, "type": "str",
                                   "choices": ["protocol-error", "profile-missing", "accounting-stop-missed",
                                               "accounting-event", "endpoint-block", "radiusd-other",
                                               "none"]},
                "rsso-log-period": {"required": False, "type": "int"},
                "rsso-radius-response": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "rsso-radius-server-port": {"required": False, "type": "int"},
                "rsso-secret": {"required": False, "type": "str"},
                "rsso-validate-request-secret": {"required": False, "type": "str",
                                                 "choices": ["enable", "disable"]},
                "secondary-secret": {"required": False, "type": "str"},
                "secondary-server": {"required": False, "type": "str"},
                "secret": {"required": False, "type": "str"},
                "server": {"required": False, "type": "str"},
                "source-ip": {"required": False, "type": "str"},
                "sso-attribute": {"required": False, "type": "str",
                                  "choices": ["User-Name", "NAS-IP-Address", "Framed-IP-Address",
                                              "Framed-IP-Netmask", "Filter-Id", "Login-IP-Host",
                                              "Reply-Message", "Callback-Number", "Callback-Id",
                                              "Framed-Route", "Framed-IPX-Network", "Class",
                                              "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier",
                                              "Proxy-State", "Login-LAT-Service", "Login-LAT-Node",
                                              "Login-LAT-Group", "Framed-AppleTalk-Zone", "Acct-Session-Id",
                                              "Acct-Multi-Session-Id"]},
                "sso-attribute-key": {"required": False, "type": "str"},
                "sso-attribute-value-override": {"required": False, "type": "str",
                                                 "choices": ["enable", "disable"]},
                "tertiary-secret": {"required": False, "type": "str"},
                "tertiary-server": {"required": False, "type": "str"},
                "timeout": {"required": False, "type": "int"},
                "use-management-vdom": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "username-case-sensitive": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_user(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Copyright: (c) 2019, Guillaume Martinez (lunik@tiwabbit.fr)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import

import pytest

from ansible.modules.source_control.gitlab_deploy_key import GitLabDeployKey


def _dummy(x):
    """Dummy function.  Only used as a placeholder for toplevel definitions when the test is going
    to be skipped anyway"""
    return x


pytestmark = []
try:
    from .gitlab import (GitlabModuleTestCase,
                         python_version_match_requirement,
                         resp_get_project, resp_find_project_deploy_key,
                         resp_create_project_deploy_key, resp_delete_project_deploy_key)

    # Gitlab module requirements
    if python_version_match_requirement():
        from gitlab.v4.objects import ProjectKey
except ImportError:
    pytestmark.append(pytest.mark.skip("Could not load gitlab module required for testing"))
    # Need to set these to something so that we don't fail when parsing
    GitlabModuleTestCase = object
    resp_get_project = _dummy
    resp_find_project_deploy_key = _dummy
    resp_create_project_deploy_key = _dummy
    resp_delete_project_deploy_key = _dummy

# Unit tests requirements
try:
    from httmock import with_httmock  # noqa
except ImportError:
    pytestmark.append(pytest.mark.skip("Could not load httmock module required for testing"))
    with_httmock = _dummy


class TestGitlabDeployKey(GitlabModuleTestCase):
    def setUp(self):
        super(TestGitlabDeployKey, self).setUp()

        self.moduleUtil = GitLabDeployKey(module=self.mock_module, gitlab_instance=self.gitlab_instance)

    @with_httmock(resp_get_project)
    @with_httmock(resp_find_project_deploy_key)
    def test_deploy_key_exist(self):
        project = self.gitlab_instance.projects.get(1)

        rvalue = self.moduleUtil.existsDeployKey(project, "Public key")

        self.assertEqual(rvalue, True)

        rvalue = self.moduleUtil.existsDeployKey(project, "Private key")

        self.assertEqual(rvalue, False)

    @with_httmock(resp_get_project)
    @with_httmock(resp_create_project_deploy_key)
    def test_create_deploy_key(self):
        project = self.gitlab_instance.projects.get(1)

        deploy_key = self.moduleUtil.createDeployKey(project, {"title": "Public key",
                                                               "key": "ssh-rsa AAAAB3NzaC1yc2EAAAABJQAAAIEAiPWx6WM"
                                                               "4lhHNedGfBpPJNPpZ7yKu+dnn1SJejgt4596k6YjzGGphH2TUxwKzxc"
                                                               "KDKKezwkpfnxPkSMkuEspGRt/aZZ9wa++Oi7Qkr8prgHc4soW6NUlfD"
                                                               "zpvZK2H5E7eQaSeP3SAwGmQKUFHCddNaP0L+hM7zhFNzjFvpaMgJw0="})

        self.assertEqual(type(deploy_key), ProjectKey)
        self.assertEqual(deploy_key.title, "Public key")

    @with_httmock(resp_get_project)
    @with_httmock(resp_find_project_deploy_key)
    @with_httmock(resp_create_project_deploy_key)
    def test_update_deploy_key(self):
        project = self.gitlab_instance.projects.get(1)
        deployKey = self.moduleUtil.findDeployKey(project, "Public key")

        changed, newDeploy_key = self.moduleUtil.updateDeployKey(deployKey, {"title": "Private key"})

        self.assertEqual(changed, True)
        self.assertEqual(type(newDeploy_key), ProjectKey)
        self.assertEqual(newDeploy_key.title, "Private key")

        changed, newDeploy_key = self.moduleUtil.updateDeployKey(deployKey, {"title": "Private key"})

        self.assertEqual(changed, False)
        self.assertEqual(newDeploy_key.title, "Private key")

    @with_httmock(resp_get_project)
    @with_httmock(resp_find_project_deploy_key)
    @with_httmock(resp_delete_project_deploy_key)
    def test_delete_deploy_key(self):
        project = self.gitlab_instance.projects.get(1)

        self.moduleUtil.existsDeployKey(project, "Public key")

        rvalue = self.moduleUtil.deleteDeployKey()

        self.assertEqual(rvalue, None)



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_router_bgp
short_description: Configure BGP in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by allowing the
      user to set and modify router feature and bgp category.
      Examples include all parameters and values need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: true
    router_bgp:
        description:
            - Configure BGP.
        default: null
        suboptions:
            admin-distance:
                description:
                    - Administrative distance modifications.
                suboptions:
                    distance:
                        description:
                            - Administrative distance to apply (1 - 255).
                    id:
                        description:
                            - ID.
                        required: true
                    neighbour-prefix:
                        description:
                            - Neighbor address prefix.
                    route-list:
                        description:
                            - Access list of routes to apply new distance to. Source router.access-list.name.
            aggregate-address:
                description:
                    - BGP aggregate address table.
                suboptions:
                    as-set:
                        description:
                            - Enable/disable generate AS set path information.
                        choices:
                            - enable
                            - disable
                    id:
                        description:
                            - ID.
                        required: true
                    prefix:
                        description:
                            - Aggregate prefix.
                    summary-only:
                        description:
                            - Enable/disable filter more specific routes from updates.
                        choices:
                            - enable
                            - disable
            aggregate-address6:
                description:
                    - BGP IPv6 aggregate address table.
                suboptions:
                    as-set:
                        description:
                            - Enable/disable generate AS set path information.
                        choices:
                            - enable
                            - disable
                    id:
                        description:
                            - ID.
                        required: true
                    prefix6:
                        description:
                            - Aggregate IPv6 prefix.
                    summary-only:
                        description:
                            - Enable/disable filter more specific routes from updates.
                        choices:
                            - enable
                            - disable
            always-compare-med:
                description:
                    - Enable/disable always compare MED.
                choices:
                    - enable
                    - disable
            as:
                description:
                    - Router AS number, valid from 1 to 4294967295, 0 to disable BGP.
            bestpath-as-path-ignore:
                description:
                    - Enable/disable ignore AS path.
                choices:
                    - enable
                    - disable
            bestpath-cmp-confed-aspath:
                description:
                    - Enable/disable compare federation AS path length.
                choices:
                    - enable
                    - disable
            bestpath-cmp-routerid:
                description:
                    - Enable/disable compare router ID for identical EBGP paths.
                choices:
                    - enable
                    - disable
            bestpath-med-confed:
                description:
                    - Enable/disable compare MED among confederation paths.
                choices:
                    - enable
                    - disable
            bestpath-med-missing-as-worst:
                description:
                    - Enable/disable treat missing MED as least preferred.
                choices:
                    - enable
                    - disable
            client-to-client-reflection:
                description:
                    - Enable/disable client-to-client route reflection.
                choices:
                    - enable
                    - disable
            cluster-id:
                description:
                    - Route reflector cluster ID.
            confederation-identifier:
                description:
                    - Confederation identifier.
            confederation-peers:
                description:
                    - Confederation peers.
                suboptions:
                    peer:
                        description:
                            - Peer ID.
                        required: true
            dampening:
                description:
                    - Enable/disable route-flap dampening.
                choices:
                    - enable
                    - disable
            dampening-max-suppress-time:
                description:
                    - Maximum minutes a route can be suppressed.
            dampening-reachability-half-life:
                description:
                    - Reachability half-life time for penalty (min).
            dampening-reuse:
                description:
                    - Threshold to reuse routes.
            dampening-route-map:
                description:
                    - Criteria for dampening. Source router.route-map.name.
            dampening-suppress:
                description:
                    - Threshold to suppress routes.
            dampening-unreachability-half-life:
                description:
                    - Unreachability half-life time for penalty (min).
            default-local-preference:
                description:
                    - Default local preference.
            deterministic-med:
                description:
                    - Enable/disable enforce deterministic comparison of MED.
                choices:
                    - enable
                    - disable
            distance-external:
                description:
                    - Distance for routes external to the AS.
            distance-internal:
                description:
                    - Distance for routes internal to the AS.
            distance-local:
                description:
                    - Distance for routes local to the AS.
            ebgp-multipath:
                description:
                    - Enable/disable EBGP multi-path.
                choices:
                    - enable
                    - disable
            enforce-first-as:
                description:
                    - Enable/disable enforce first AS for EBGP routes.
                choices:
                    - enable
                    - disable
            fast-external-failover:
                description:
                    - Enable/disable reset peer BGP session if link goes down.
                choices:
                    - enable
                    - disable
            graceful-end-on-timer:
                description:
                    - Enable/disable to exit graceful restart on timer only.
                choices:
                    - enable
                    - disable
            graceful-restart:
                description:
                    - Enable/disable BGP graceful restart capabilities.
                choices:
                    - enable
                    - disable
            graceful-restart-time:
                description:
                    - Time needed for neighbors to restart (sec).
            graceful-stalepath-time:
                description:
                    - Time to hold stale paths of restarting neighbor (sec).
            graceful-update-delay:
                description:
                    - Route advertisement/selection delay after restart (sec).
            holdtime-timer:
                description:
                    - Number of seconds to mark peer as dead.
            ibgp-multipath:
                description:
                    - Enable/disable IBGP multi-path.
                choices:
                    - enable
                    - disable
            ignore-optional-capability:
                description:
                    - Don't send unknown optional capability notification message
                choices:
                    - enable
                    - disable
            keepalive-timer:
                description:
                    - Frequency to send keep alive requests.
            log-neighbour-changes:
                description:
                    - Enable logging of BGP neighbour's changes
                choices:
                    - enable
                    - disable
            neighbor:
                description:
                    - BGP neighbor table.
                suboptions:
                    activate:
                        description:
                            - Enable/disable address family IPv4 for this neighbor.
                        choices:
                            - enable
                            - disable
                    activate6:
                        description:
                            - Enable/disable address family IPv6 for this neighbor.
                        choices:
                            - enable
                            - disable
                    advertisement-interval:
                        description:
                            - Minimum interval (sec) between sending updates.
                    allowas-in:
                        description:
                            - IPv4 The maximum number of occurrence of my AS number allowed.
                    allowas-in-enable:
                        description:
                            - Enable/disable IPv4 Enable to allow my AS in AS path.
                        choices:
                            - enable
                            - disable
                    allowas-in-enable6:
                        description:
                            - Enable/disable IPv6 Enable to allow my AS in AS path.
                        choices:
                            - enable
                            - disable
                    allowas-in6:
                        description:
                            - IPv6 The maximum number of occurrence of my AS number allowed.
                    as-override:
                        description:
                            - Enable/disable replace peer AS with own AS for IPv4.
                        choices:
                            - enable
                            - disable
                    as-override6:
                        description:
                            - Enable/disable replace peer AS with own AS for IPv6.
                        choices:
                            - enable
                            - disable
                    attribute-unchanged:
                        description:
                            - IPv4 List of attributes that should be unchanged.
                        choices:
                            - as-path
                            - med
                            - next-hop
                    attribute-unchanged6:
                        description:
                            - IPv6 List of attributes that should be unchanged.
                        choices:
                            - as-path
                            - med
                            - next-hop
                    bfd:
                        description:
                            - Enable/disable BFD for this neighbor.
                        choices:
                            - enable
                            - disable
                    capability-default-originate:
                        description:
                            - Enable/disable advertise default IPv4 route to this neighbor.
                        choices:
                            - enable
                            - disable
                    capability-default-originate6:
                        description:
                            - Enable/disable advertise default IPv6 route to this neighbor.
                        choices:
                            - enable
                            - disable
                    capability-dynamic:
                        description:
                            - Enable/disable advertise dynamic capability to this neighbor.
                        choices:
                            - enable
                            - disable
                    capability-graceful-restart:
                        description:
                            - Enable/disable advertise IPv4 graceful restart capability to this neighbor.
                        choices:
                            - enable
                            - disable
                    capability-graceful-restart6:
                        description:
                            - Enable/disable advertise IPv6 graceful restart capability to this neighbor.
                        choices:
                            - enable
                            - disable
                    capability-orf:
                        description:
                            - Accept/Send IPv4 ORF lists to/from this neighbor.
                        choices:
                            - none
                            - receive
                            - send
                            - both
                    capability-orf6:
                        description:
                            - Accept/Send IPv6 ORF lists to/from this neighbor.
                        choices:
                            - none
                            - receive
                            - send
                            - both
                    capability-route-refresh:
                        description:
                            - Enable/disable advertise route refresh capability to this neighbor.
                        choices:
                            - enable
                            - disable
                    conditional-advertise:
                        description:
                            - Conditional advertisement.
                        suboptions:
                            advertise-routemap:
                                description:
                                    - Name of advertising route map. Source router.route-map.name.
                                required: true
                            condition-routemap:
                                description:
                                    - Name of condition route map. Source router.route-map.name.
                            condition-type:
                                description:
                                    - Type of condition.
                                choices:
                                    - exist
                                    - non-exist
                    connect-timer:
                        description:
                            - Interval (sec) for connect timer.
                    default-originate-routemap:
                        description:
                            - Route map to specify criteria to originate IPv4 default. Source router.route-map.name.
                    default-originate-routemap6:
                        description:
                            - Route map to specify criteria to originate IPv6 default. Source router.route-map.name.
                    description:
                        description:
                            - Description.
                    distribute-list-in:
                        description:
                            - Filter for IPv4 updates from this neighbor. Source router.access-list.name.
                    distribute-list-in6:
                        description:
                            - Filter for IPv6 updates from this neighbor. Source router.access-list6.name.
                    distribute-list-out:
                        description:
                            - Filter for IPv4 updates to this neighbor. Source router.access-list.name.
                    distribute-list-out6:
                        description:
                            - Filter for IPv6 updates to this neighbor. Source router.access-list6.name.
                    dont-capability-negotiate:
                        description:
                            - Don't negotiate capabilities with this neighbor
                        choices:
                            - enable
                            - disable
                    ebgp-enforce-multihop:
                        description:
                            - Enable/disable allow multi-hop EBGP neighbors.
                        choices:
                            - enable
                            - disable
                    ebgp-multihop-ttl:
                        description:
                            - EBGP multihop TTL for this peer.
                    filter-list-in:
                        description:
                            - BGP filter for IPv4 inbound routes. Source router.aspath-list.name.
                    filter-list-in6:
                        description:
                            - BGP filter for IPv6 inbound routes. Source router.aspath-list.name.
                    filter-list-out:
                        description:
                            - BGP filter for IPv4 outbound routes. Source router.aspath-list.name.
                    filter-list-out6:
                        description:
                            - BGP filter for IPv6 outbound routes. Source router.aspath-list.name.
                    holdtime-timer:
                        description:
                            - Interval (sec) before peer considered dead.
                    interface:
                        description:
                            - Interface Source system.interface.name.
                    ip:
                        description:
                            - IP/IPv6 address of neighbor.
                        required: true
                    keep-alive-timer:
                        description:
                            - Keep alive timer interval (sec).
                    link-down-failover:
                        description:
                            - Enable/disable failover upon link down.
                        choices:
                            - enable
                            - disable
                    local-as:
                        description:
                            - Local AS number of neighbor.
                    local-as-no-prepend:
                        description:
                            - Do not prepend local-as to incoming updates.
                        choices:
                            - enable
                            - disable
                    local-as-replace-as:
                        description:
                            - Replace real AS with local-as in outgoing updates.
                        choices:
                            - enable
                            - disable
                    maximum-prefix:
                        description:
                            - Maximum number of IPv4 prefixes to accept from this peer.
                    maximum-prefix-threshold:
                        description:
                            - Maximum IPv4 prefix threshold value (1 - 100 percent).
                    maximum-prefix-threshold6:
                        description:
                            - Maximum IPv6 prefix threshold value (1 - 100 percent).
                    maximum-prefix-warning-only:
                        description:
                            - Enable/disable IPv4 Only give warning message when limit is exceeded.
                        choices:
                            - enable
                            - disable
                    maximum-prefix-warning-only6:
                        description:
                            - Enable/disable IPv6 Only give warning message when limit is exceeded.
                        choices:
                            - enable
                            - disable
                    maximum-prefix6:
                        description:
                            - Maximum number of IPv6 prefixes to accept from this peer.
                    next-hop-self:
                        description:
                            - Enable/disable IPv4 next-hop calculation for this neighbor.
                        choices:
                            - enable
                            - disable
                    next-hop-self6:
                        description:
                            - Enable/disable IPv6 next-hop calculation for this neighbor.
                        choices:
                            - enable
                            - disable
                    override-capability:
                        description:
                            - Enable/disable override result of capability negotiation.
                        choices:
                            - enable
                            - disable
                    passive:
                        description:
                            - Enable/disable sending of open messages to this neighbor.
                        choices:
                            - enable
                            - disable
                    password:
                        description:
                            - Password used in MD5 authentication.
                    prefix-list-in:
                        description:
                            - IPv4 Inbound filter for updates from this neighbor. Source router.prefix-list.name.
                    prefix-list-in6:
                        description:
                            - IPv6 Inbound filter for updates from this neighbor. Source router.prefix-list6.name.
                    prefix-list-out:
                        description:
                            - IPv4 Outbound filter for updates to this neighbor. Source router.prefix-list.name.
                    prefix-list-out6:
                        description:
                            - IPv6 Outbound filter for updates to this neighbor. Source router.prefix-list6.name.
                    remote-as:
                        description:
                            - AS number of neighbor.
                    remove-private-as:
                        description:
                            - Enable/disable remove private AS number from IPv4 outbound updates.
                        choices:
                            - enable
                            - disable
                    remove-private-as6:
                        description:
                            - Enable/disable remove private AS number from IPv6 outbound updates.
                        choices:
                            - enable
                            - disable
                    restart-time:
                        description:
                            - Graceful restart delay time (sec, 0 = global default).
                    retain-stale-time:
                        description:
                            - Time to retain stale routes.
                    route-map-in:
                        description:
                            - IPv4 Inbound route map filter. Source router.route-map.name.
                    route-map-in6:
                        description:
                            - IPv6 Inbound route map filter. Source router.route-map.name.
                    route-map-out:
                        description:
                            - IPv4 Outbound route map filter. Source router.route-map.name.
                    route-map-out6:
                        description:
                            - IPv6 Outbound route map filter. Source router.route-map.name.
                    route-reflector-client:
                        description:
                            - Enable/disable IPv4 AS route reflector client.
                        choices:
                            - enable
                            - disable
                    route-reflector-client6:
                        description:
                            - Enable/disable IPv6 AS route reflector client.
                        choices:
                            - enable
                            - disable
                    route-server-client:
                        description:
                            - Enable/disable IPv4 AS route server client.
                        choices:
                            - enable
                            - disable
                    route-server-client6:
                        description:
                            - Enable/disable IPv6 AS route server client.
                        choices:
                            - enable
                            - disable
                    send-community:
                        description:
                            - IPv4 Send community attribute to neighbor.
                        choices:
                            - standard
                            - extended
                            - both
                            - disable
                    send-community6:
                        description:
                            - IPv6 Send community attribute to neighbor.
                        choices:
                            - standard
                            - extended
                            - both
                            - disable
                    shutdown:
                        description:
                            - Enable/disable shutdown this neighbor.
                        choices:
                            - enable
                            - disable
                    soft-reconfiguration:
                        description:
                            - Enable/disable allow IPv4 inbound soft reconfiguration.
                        choices:
                            - enable
                            - disable
                    soft-reconfiguration6:
                        description:
                            - Enable/disable allow IPv6 inbound soft reconfiguration.
                        choices:
                            - enable
                            - disable
                    stale-route:
                        description:
                            - Enable/disable stale route after neighbor down.
                        choices:
                            - enable
                            - disable
                    strict-capability-match:
                        description:
                            - Enable/disable strict capability matching.
                        choices:
                            - enable
                            - disable
                    unsuppress-map:
                        description:
                            - IPv4 Route map to selectively unsuppress suppressed routes. Source router.route-map.name.
                    unsuppress-map6:
                        description:
                            - IPv6 Route map to selectively unsuppress suppressed routes. Source router.route-map.name.
                    update-source:
                        description:
                            - Interface to use as source IP/IPv6 address of TCP connections. Source system.interface.name.
                    weight:
                        description:
                            - Neighbor weight.
            neighbor-group:
                description:
                    - BGP neighbor group table.
                suboptions:
                    activate:
                        description:
                            - Enable/disable address family IPv4 for this neighbor.
                        choices:
                            - enable
                            - disable
                    activate6:
                        description:
                            - Enable/disable address family IPv6 for this neighbor.
                        choices:
                            - enable
                            - disable
                    advertisement-interval:
                        description:
                            - Minimum interval (sec) between sending updates.
                    allowas-in:
                        description:
                            - IPv4 The maximum number of occurrence of my AS number allowed.
                    allowas-in-enable:
                        description:
                            - Enable/disable IPv4 Enable to allow my AS in AS path.
                        choices:
                            - enable
                            - disable
                    allowas-in-enable6:
                        description:
                            - Enable/disable IPv6 Enable to allow my AS in AS path.
                        choices:
                            - enable
                            - disable
                    allowas-in6:
                        description:
                            - IPv6 The maximum number of occurrence of my AS number allowed.
                    as-override:
                        description:
                            - Enable/disable replace peer AS with own AS for IPv4.
                        choices:
                            - enable
                            - disable
                    as-override6:
                        description:
                            - Enable/disable replace peer AS with own AS for IPv6.
                        choices:
                            - enable
                            - disable
                    attribute-unchanged:
                        description:
                            - IPv4 List of attributes that should be unchanged.
                        choices:
                            - as-path
                            - med
                            - next-hop
                    attribute-unchanged6:
                        description:
                            - IPv6 List of attributes that should be unchanged.
                        choices:
                            - as-path
                            - med
                            - next-hop
                    bfd:
                        description:
                            - Enable/disable BFD for this neighbor.
                        choices:
                            - enable
                            - disable
                    capability-default-originate:
                        description:
                            - Enable/disable advertise default IPv4 route to this neighbor.
                        choices:
                            - enable
                            - disable
                    capability-default-originate6:
                        description:
                            - Enable/disable advertise default IPv6 route to this neighbor.
                        choices:
                            - enable
                            - disable
                    capability-dynamic:
                        description:
                            - Enable/disable advertise dynamic capability to this neighbor.
                        choices:
                            - enable
                            - disable
                    capability-graceful-restart:
                        description:
                            - Enable/disable advertise IPv4 graceful restart capability to this neighbor.
                        choices:
                            - enable
                            - disable
                    capability-graceful-restart6:
                        description:
                            - Enable/disable advertise IPv6 graceful restart capability to this neighbor.
                        choices:
                            - enable
                            - disable
                    capability-orf:
                        description:
                            - Accept/Send IPv4 ORF lists to/from this neighbor.
                        choices:
                            - none
                            - receive
                            - send
                            - both
                    capability-orf6:
                        description:
                            - Accept/Send IPv6 ORF lists to/from this neighbor.
                        choices:
                            - none
                            - receive
                            - send
                            - both
                    capability-route-refresh:
                        description:
                            - Enable/disable advertise route refresh capability to this neighbor.
                        choices:
                            - enable
                            - disable
                    connect-timer:
                        description:
                            - Interval (sec) for connect timer.
                    default-originate-routemap:
                        description:
                            - Route map to specify criteria to originate IPv4 default. Source router.route-map.name.
                    default-originate-routemap6:
                        description:
                            - Route map to specify criteria to originate IPv6 default. Source router.route-map.name.
                    description:
                        description:
                            - Description.
                    distribute-list-in:
                        description:
                            - Filter for IPv4 updates from this neighbor. Source router.access-list.name.
                    distribute-list-in6:
                        description:
                            - Filter for IPv6 updates from this neighbor. Source router.access-list6.name.
                    distribute-list-out:
                        description:
                            - Filter for IPv4 updates to this neighbor. Source router.access-list.name.
                    distribute-list-out6:
                        description:
                            - Filter for IPv6 updates to this neighbor. Source router.access-list6.name.
                    dont-capability-negotiate:
                        description:
                            - Don't negotiate capabilities with this neighbor
                        choices:
                            - enable
                            - disable
                    ebgp-enforce-multihop:
                        description:
                            - Enable/disable allow multi-hop EBGP neighbors.
                        choices:
                            - enable
                            - disable
                    ebgp-multihop-ttl:
                        description:
                            - EBGP multihop TTL for this peer.
                    filter-list-in:
                        description:
                            - BGP filter for IPv4 inbound routes. Source router.aspath-list.name.
                    filter-list-in6:
                        description:
                            - BGP filter for IPv6 inbound routes. Source router.aspath-list.name.
                    filter-list-out:
                        description:
                            - BGP filter for IPv4 outbound routes. Source router.aspath-list.name.
                    filter-list-out6:
                        description:
                            - BGP filter for IPv6 outbound routes. Source router.aspath-list.name.
                    holdtime-timer:
                        description:
                            - Interval (sec) before peer considered dead.
                    interface:
                        description:
                            - Interface Source system.interface.name.
                    keep-alive-timer:
                        description:
                            - Keep alive timer interval (sec).
                    link-down-failover:
                        description:
                            - Enable/disable failover upon link down.
                        choices:
                            - enable
                            - disable
                    local-as:
                        description:
                            - Local AS number of neighbor.
                    local-as-no-prepend:
                        description:
                            - Do not prepend local-as to incoming updates.
                        choices:
                            - enable
                            - disable
                    local-as-replace-as:
                        description:
                            - Replace real AS with local-as in outgoing updates.
                        choices:
                            - enable
                            - disable
                    maximum-prefix:
                        description:
                            - Maximum number of IPv4 prefixes to accept from this peer.
                    maximum-prefix-threshold:
                        description:
                            - Maximum IPv4 prefix threshold value (1 - 100 percent).
                    maximum-prefix-threshold6:
                        description:
                            - Maximum IPv6 prefix threshold value (1 - 100 percent).
                    maximum-prefix-warning-only:
                        description:
                            - Enable/disable IPv4 Only give warning message when limit is exceeded.
                        choices:
                            - enable
                            - disable
                    maximum-prefix-warning-only6:
                        description:
                            - Enable/disable IPv6 Only give warning message when limit is exceeded.
                        choices:
                            - enable
                            - disable
                    maximum-prefix6:
                        description:
                            - Maximum number of IPv6 prefixes to accept from this peer.
                    name:
                        description:
                            - Neighbor group name.
                        required: true
                    next-hop-self:
                        description:
                            - Enable/disable IPv4 next-hop calculation for this neighbor.
                        choices:
                            - enable
                            - disable
                    next-hop-self6:
                        description:
                            - Enable/disable IPv6 next-hop calculation for this neighbor.
                        choices:
                            - enable
                            - disable
                    override-capability:
                        description:
                            - Enable/disable override result of capability negotiation.
                        choices:
                            - enable
                            - disable
                    passive:
                        description:
                            - Enable/disable sending of open messages to this neighbor.
                        choices:
                            - enable
                            - disable
                    prefix-list-in:
                        description:
                            - IPv4 Inbound filter for updates from this neighbor. Source router.prefix-list.name.
                    prefix-list-in6:
                        description:
                            - IPv6 Inbound filter for updates from this neighbor. Source router.prefix-list6.name.
                    prefix-list-out:
                        description:
                            - IPv4 Outbound filter for updates to this neighbor. Source router.prefix-list.name.
                    prefix-list-out6:
                        description:
                            - IPv6 Outbound filter for updates to this neighbor. Source router.prefix-list6.name.
                    remote-as:
                        description:
                            - AS number of neighbor.
                    remove-private-as:
                        description:
                            - Enable/disable remove private AS number from IPv4 outbound updates.
                        choices:
                            - enable
                            - disable
                    remove-private-as6:
                        description:
                            - Enable/disable remove private AS number from IPv6 outbound updates.
                        choices:
                            - enable
                            - disable
                    restart-time:
                        description:
                            - Graceful restart delay time (sec, 0 = global default).
                    retain-stale-time:
                        description:
                            - Time to retain stale routes.
                    route-map-in:
                        description:
                            - IPv4 Inbound route map filter. Source router.route-map.name.
                    route-map-in6:
                        description:
                            - IPv6 Inbound route map filter. Source router.route-map.name.
                    route-map-out:
                        description:
                            - IPv4 Outbound route map filter. Source router.route-map.name.
                    route-map-out6:
                        description:
                            - IPv6 Outbound route map filter. Source router.route-map.name.
                    route-reflector-client:
                        description:
                            - Enable/disable IPv4 AS route reflector client.
                        choices:
                            - enable
                            - disable
                    route-reflector-client6:
                        description:
                            - Enable/disable IPv6 AS route reflector client.
                        choices:
                            - enable
                            - disable
                    route-server-client:
                        description:
                            - Enable/disable IPv4 AS route server client.
                        choices:
                            - enable
                            - disable
                    route-server-client6:
                        description:
                            - Enable/disable IPv6 AS route server client.
                        choices:
                            - enable
                            - disable
                    send-community:
                        description:
                            - IPv4 Send community attribute to neighbor.
                        choices:
                            - standard
                            - extended
                            - both
                            - disable
                    send-community6:
                        description:
                            - IPv6 Send community attribute to neighbor.
                        choices:
                            - standard
                            - extended
                            - both
                            - disable
                    shutdown:
                        description:
                            - Enable/disable shutdown this neighbor.
                        choices:
                            - enable
                            - disable
                    soft-reconfiguration:
                        description:
                            - Enable/disable allow IPv4 inbound soft reconfiguration.
                        choices:
                            - enable
                            - disable
                    soft-reconfiguration6:
                        description:
                            - Enable/disable allow IPv6 inbound soft reconfiguration.
                        choices:
                            - enable
                            - disable
                    stale-route:
                        description:
                            - Enable/disable stale route after neighbor down.
                        choices:
                            - enable
                            - disable
                    strict-capability-match:
                        description:
                            - Enable/disable strict capability matching.
                        choices:
                            - enable
                            - disable
                    unsuppress-map:
                        description:
                            - IPv4 Route map to selectively unsuppress suppressed routes. Source router.route-map.name.
                    unsuppress-map6:
                        description:
                            - IPv6 Route map to selectively unsuppress suppressed routes. Source router.route-map.name.
                    update-source:
                        description:
                            - Interface to use as source IP/IPv6 address of TCP connections. Source system.interface.name.
                    weight:
                        description:
                            - Neighbor weight.
            neighbor-range:
                description:
                    - BGP neighbor range table.
                suboptions:
                    id:
                        description:
                            - Neighbor range ID.
                        required: true
                    max-neighbor-num:
                        description:
                            - Maximum number of neighbors.
                    neighbor-group:
                        description:
                            - Neighbor group name. Source router.bgp.neighbor-group.name.
                    prefix:
                        description:
                            - Neighbor range prefix.
            network:
                description:
                    - BGP network table.
                suboptions:
                    backdoor:
                        description:
                            - Enable/disable route as backdoor.
                        choices:
                            - enable
                            - disable
                    id:
                        description:
                            - ID.
                        required: true
                    prefix:
                        description:
                            - Network prefix.
                    route-map:
                        description:
                            - Route map to modify generated route. Source router.route-map.name.
            network-import-check:
                description:
                    - Enable/disable ensure BGP network route exists in IGP.
                choices:
                    - enable
                    - disable
            network6:
                description:
                    - BGP IPv6 network table.
                suboptions:
                    backdoor:
                        description:
                            - Enable/disable route as backdoor.
                        choices:
                            - enable
                            - disable
                    id:
                        description:
                            - ID.
                        required: true
                    prefix6:
                        description:
                            - Network IPv6 prefix.
                    route-map:
                        description:
                            - Route map to modify generated route. Source router.route-map.name.
            redistribute:
                description:
                    - BGP IPv4 redistribute table.
                suboptions:
                    name:
                        description:
                            - Distribute list entry name.
                        required: true
                    route-map:
                        description:
                            - Route map name. Source router.route-map.name.
                    status:
                        description:
                            - Status
                        choices:
                            - enable
                            - disable
            redistribute6:
                description:
                    - BGP IPv6 redistribute table.
                suboptions:
                    name:
                        description:
                            - Distribute list entry name.
                        required: true
                    route-map:
                        description:
                            - Route map name. Source router.route-map.name.
                    status:
                        description:
                            - Status
                        choices:
                            - enable
                            - disable
            router-id:
                description:
                    - Router ID.
            scan-time:
                description:
                    - Background scanner interval (sec), 0 to disable it.
            synchronization:
                description:
                    - Enable/disable only advertise routes from iBGP if routes present in an IGP.
                choices:
                    - enable
                    - disable
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure BGP.
    fortios_router_bgp:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      https: "False"
      router_bgp:
        admin-distance:
         -
            distance: "4"
            id:  "5"
            neighbour-prefix: "<your_own_value>"
            route-list: "<your_own_value> (source router.access-list.name)"
        aggregate-address:
         -
            as-set: "enable"
            id:  "10"
            prefix: "<your_own_value>"
            summary-only: "enable"
        aggregate-address6:
         -
            as-set: "enable"
            id:  "15"
            prefix6: "<your_own_value>"
            summary-only: "enable"
        always-compare-med: "enable"
        as: "19"
        bestpath-as-path-ignore: "enable"
        bestpath-cmp-confed-aspath: "enable"
        bestpath-cmp-routerid: "enable"
        bestpath-med-confed: "enable"
        bestpath-med-missing-as-worst: "enable"
        client-to-client-reflection: "enable"
        cluster-id: "<your_own_value>"
        confederation-identifier: "27"
        confederation-peers:
         -
            peer: "<your_own_value>"
        dampening: "enable"
        dampening-max-suppress-time: "31"
        dampening-reachability-half-life: "32"
        dampening-reuse: "33"
        dampening-route-map: "<your_own_value> (source router.route-map.name)"
        dampening-suppress: "35"
        dampening-unreachability-half-life: "36"
        default-local-preference: "37"
        deterministic-med: "enable"
        distance-external: "39"
        distance-internal: "40"
        distance-local: "41"
        ebgp-multipath: "enable"
        enforce-first-as: "enable"
        fast-external-failover: "enable"
        graceful-end-on-timer: "enable"
        graceful-restart: "enable"
        graceful-restart-time: "47"
        graceful-stalepath-time: "48"
        graceful-update-delay: "49"
        holdtime-timer: "50"
        ibgp-multipath: "enable"
        ignore-optional-capability: "enable"
        keepalive-timer: "53"
        log-neighbour-changes: "enable"
        neighbor:
         -
            activate: "enable"
            activate6: "enable"
            advertisement-interval: "58"
            allowas-in: "59"
            allowas-in-enable: "enable"
            allowas-in-enable6: "enable"
            allowas-in6: "62"
            as-override: "enable"
            as-override6: "enable"
            attribute-unchanged: "as-path"
            attribute-unchanged6: "as-path"
            bfd: "enable"
            capability-default-originate: "enable"
            capability-default-originate6: "enable"
            capability-dynamic: "enable"
            capability-graceful-restart: "enable"
            capability-graceful-restart6: "enable"
            capability-orf: "none"
            capability-orf6: "none"
            capability-route-refresh: "enable"
            conditional-advertise:
             -
                advertise-routemap: "<your_own_value> (source router.route-map.name)"
                condition-routemap: "<your_own_value> (source router.route-map.name)"
                condition-type: "exist"
            connect-timer: "80"
            default-originate-routemap: "<your_own_value> (source router.route-map.name)"
            default-originate-routemap6: "<your_own_value> (source router.route-map.name)"
            description: "<your_own_value>"
            distribute-list-in: "<your_own_value> (source router.access-list.name)"
            distribute-list-in6: "<your_own_value> (source router.access-list6.name)"
            distribute-list-out: "<your_own_value> (source router.access-list.name)"
            distribute-list-out6: "<your_own_value> (source router.access-list6.name)"
            dont-capability-negotiate: "enable"
            ebgp-enforce-multihop: "enable"
            ebgp-multihop-ttl: "90"
            filter-list-in: "<your_own_value> (source router.aspath-list.name)"
            filter-list-in6: "<your_own_value> (source router.aspath-list.name)"
            filter-list-out: "<your_own_value> (source router.aspath-list.name)"
            filter-list-out6: "<your_own_value> (source router.aspath-list.name)"
            holdtime-timer: "95"
            interface: "<your_own_value> (source system.interface.name)"
            ip: "<your_own_value>"
            keep-alive-timer: "98"
            link-down-failover: "enable"
            local-as: "100"
            local-as-no-prepend: "enable"
            local-as-replace-as: "enable"
            maximum-prefix: "103"
            maximum-prefix-threshold: "104"
            maximum-prefix-threshold6: "105"
            maximum-prefix-warning-only: "enable"
            maximum-prefix-warning-only6: "enable"
            maximum-prefix6: "108"
            next-hop-self: "enable"
            next-hop-self6: "enable"
            override-capability: "enable"
            passive: "enable"
            password: "<your_own_value>"
            prefix-list-in: "<your_own_value> (source router.prefix-list.name)"
            prefix-list-in6: "<your_own_value> (source router.prefix-list6.name)"
            prefix-list-out: "<your_own_value> (source router.prefix-list.name)"
            prefix-list-out6: "<your_own_value> (source router.prefix-list6.name)"
            remote-as: "118"
            remove-private-as: "enable"
            remove-private-as6: "enable"
            restart-time: "121"
            retain-stale-time: "122"
            route-map-in: "<your_own_value> (source router.route-map.name)"
            route-map-in6: "<your_own_value> (source router.route-map.name)"
            route-map-out: "<your_own_value> (source router.route-map.name)"
            route-map-out6: "<your_own_value> (source router.route-map.name)"
            route-reflector-client: "enable"
            route-reflector-client6: "enable"
            route-server-client: "enable"
            route-server-client6: "enable"
            send-community: "standard"
            send-community6: "standard"
            shutdown: "enable"
            soft-reconfiguration: "enable"
            soft-reconfiguration6: "enable"
            stale-route: "enable"
            strict-capability-match: "enable"
            unsuppress-map: "<your_own_value> (source router.route-map.name)"
            unsuppress-map6: "<your_own_value> (source router.route-map.name)"
            update-source: "<your_own_value> (source system.interface.name)"
            weight: "141"
        neighbor-group:
         -
            activate: "enable"
            activate6: "enable"
            advertisement-interval: "145"
            allowas-in: "146"
            allowas-in-enable: "enable"
            allowas-in-enable6: "enable"
            allowas-in6: "149"
            as-override: "enable"
            as-override6: "enable"
            attribute-unchanged: "as-path"
            attribute-unchanged6: "as-path"
            bfd: "enable"
            capability-default-originate: "enable"
            capability-default-originate6: "enable"
            capability-dynamic: "enable"
            capability-graceful-restart: "enable"
            capability-graceful-restart6: "enable"
            capability-orf: "none"
            capability-orf6: "none"
            capability-route-refresh: "enable"
            connect-timer: "163"
            default-originate-routemap: "<your_own_value> (source router.route-map.name)"
            default-originate-routemap6: "<your_own_value> (source router.route-map.name)"
            description: "<your_own_value>"
            distribute-list-in: "<your_own_value> (source router.access-list.name)"
            distribute-list-in6: "<your_own_value> (source router.access-list6.name)"
            distribute-list-out: "<your_own_value> (source router.access-list.name)"
            distribute-list-out6: "<your_own_value> (source router.access-list6.name)"
            dont-capability-negotiate: "enable"
            ebgp-enforce-multihop: "enable"
            ebgp-multihop-ttl: "173"
            filter-list-in: "<your_own_value> (source router.aspath-list.name)"
            filter-list-in6: "<your_own_value> (source router.aspath-list.name)"
            filter-list-out: "<your_own_value> (source router.aspath-list.name)"
            filter-list-out6: "<your_own_value> (source router.aspath-list.name)"
            holdtime-timer: "178"
            interface: "<your_own_value> (source system.interface.name)"
            keep-alive-timer: "180"
            link-down-failover: "enable"
            local-as: "182"
            local-as-no-prepend: "enable"
            local-as-replace-as: "enable"
            maximum-prefix: "185"
            maximum-prefix-threshold: "186"
            maximum-prefix-threshold6: "187"
            maximum-prefix-warning-only: "enable"
            maximum-prefix-warning-only6: "enable"
            maximum-prefix6: "190"
            name: "default_name_191"
            next-hop-self: "enable"
            next-hop-self6: "enable"
            override-capability: "enable"
            passive: "enable"
            prefix-list-in: "<your_own_value> (source router.prefix-list.name)"
            prefix-list-in6: "<your_own_value> (source router.prefix-list6.name)"
            prefix-list-out: "<your_own_value> (source router.prefix-list.name)"
            prefix-list-out6: "<your_own_value> (source router.prefix-list6.name)"
            remote-as: "200"
            remove-private-as: "enable"
            remove-private-as6: "enable"
            restart-time: "203"
            retain-stale-time: "204"
            route-map-in: "<your_own_value> (source router.route-map.name)"
            route-map-in6: "<your_own_value> (source router.route-map.name)"
            route-map-out: "<your_own_value> (source router.route-map.name)"
            route-map-out6: "<your_own_value> (source router.route-map.name)"
            route-reflector-client: "enable"
            route-reflector-client6: "enable"
            route-server-client: "enable"
            route-server-client6: "enable"
            send-community: "standard"
            send-community6: "standard"
            shutdown: "enable"
            soft-reconfiguration: "enable"
            soft-reconfiguration6: "enable"
            stale-route: "enable"
            strict-capability-match: "enable"
            unsuppress-map: "<your_own_value> (source router.route-map.name)"
            unsuppress-map6: "<your_own_value> (source router.route-map.name)"
            update-source: "<your_own_value> (source system.interface.name)"
            weight: "223"
        neighbor-range:
         -
            id:  "225"
            max-neighbor-num: "226"
            neighbor-group: "<your_own_value> (source router.bgp.neighbor-group.name)"
            prefix: "<your_own_value>"
        network:
         -
            backdoor: "enable"
            id:  "231"
            prefix: "<your_own_value>"
            route-map: "<your_own_value> (source router.route-map.name)"
        network-import-check: "enable"
        network6:
         -
            backdoor: "enable"
            id:  "237"
            prefix6: "<your_own_value>"
            route-map: "<your_own_value> (source router.route-map.name)"
        redistribute:
         -
            name: "default_name_241"
            route-map: "<your_own_value> (source router.route-map.name)"
            status: "enable"
        redistribute6:
         -
            name: "default_name_245"
            route-map: "<your_own_value> (source router.route-map.name)"
            status: "enable"
        router-id: "<your_own_value>"
        scan-time: "249"
        synchronization: "enable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_router_bgp_data(json):
    option_list = ['admin-distance', 'aggregate-address', 'aggregate-address6',
                   'always-compare-med', 'as', 'bestpath-as-path-ignore',
                   'bestpath-cmp-confed-aspath', 'bestpath-cmp-routerid', 'bestpath-med-confed',
                   'bestpath-med-missing-as-worst', 'client-to-client-reflection', 'cluster-id',
                   'confederation-identifier', 'confederation-peers', 'dampening',
                   'dampening-max-suppress-time', 'dampening-reachability-half-life', 'dampening-reuse',
                   'dampening-route-map', 'dampening-suppress', 'dampening-unreachability-half-life',
                   'default-local-preference', 'deterministic-med', 'distance-external',
                   'distance-internal', 'distance-local', 'ebgp-multipath',
                   'enforce-first-as', 'fast-external-failover', 'graceful-end-on-timer',
                   'graceful-restart', 'graceful-restart-time', 'graceful-stalepath-time',
                   'graceful-update-delay', 'holdtime-timer', 'ibgp-multipath',
                   'ignore-optional-capability', 'keepalive-timer', 'log-neighbour-changes',
                   'neighbor', 'neighbor-group', 'neighbor-range',
                   'network', 'network-import-check', 'network6',
                   'redistribute', 'redistribute6', 'router-id',
                   'scan-time', 'synchronization']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def flatten_multilists_attributes(data):
    multilist_attrs = []

    for attr in multilist_attrs:
        try:
            path = "data['" + "']['".join(elem for elem in attr) + "']"
            current_val = eval(path)
            flattened_val = ' '.join(elem for elem in current_val)
            exec(path + '= flattened_val')
        except BaseException:
            pass

    return data


def router_bgp(data, fos):
    vdom = data['vdom']
    router_bgp_data = data['router_bgp']
    flattened_data = flatten_multilists_attributes(router_bgp_data)
    filtered_data = filter_router_bgp_data(flattened_data)
    return fos.set('router',
                   'bgp',
                   data=filtered_data,
                   vdom=vdom)


def fortios_router(data, fos):
    login(data)

    if data['router_bgp']:
        resp = router_bgp(data, fos)

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": True},
        "router_bgp": {
            "required": False, "type": "dict",
            "options": {
                "admin-distance": {"required": False, "type": "list",
                                   "options": {
                                       "distance": {"required": False, "type": "int"},
                                       "id": {"required": True, "type": "int"},
                                       "neighbour-prefix": {"required": False, "type": "str"},
                                       "route-list": {"required": False, "type": "str"}
                                   }},
                "aggregate-address": {"required": False, "type": "list",
                                      "options": {
                                          "as-set": {"required": False, "type": "str",
                                                     "choices": ["enable", "disable"]},
                                          "id": {"required": True, "type": "int"},
                                          "prefix": {"required": False, "type": "str"},
                                          "summary-only": {"required": False, "type": "str",
                                                           "choices": ["enable", "disable"]}
                                      }},
                "aggregate-address6": {"required": False, "type": "list",
                                       "options": {
                                           "as-set": {"required": False, "type": "str",
                                                      "choices": ["enable", "disable"]},
                                           "id": {"required": True, "type": "int"},
                                           "prefix6": {"required": False, "type": "str"},
                                           "summary-only": {"required": False, "type": "str",
                                                            "choices": ["enable", "disable"]}
                                       }},
                "always-compare-med": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "as": {"required": False, "type": "int"},
                "bestpath-as-path-ignore": {"required": False, "type": "str",
                                            "choices": ["enable", "disable"]},
                "bestpath-cmp-confed-aspath": {"required": False, "type": "str",
                                               "choices": ["enable", "disable"]},
                "bestpath-cmp-routerid": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                "bestpath-med-confed": {"required": False, "type": "str",
                                        "choices": ["enable", "disable"]},
                "bestpath-med-missing-as-worst": {"required": False, "type": "str",
                                                  "choices": ["enable", "disable"]},
                "client-to-client-reflection": {"required": False, "type": "str",
                                                "choices": ["enable", "disable"]},
                "cluster-id": {"required": False, "type": "str"},
                "confederation-identifier": {"required": False, "type": "int"},
                "confederation-peers": {"required": False, "type": "list",
                                        "options": {
                                            "peer": {"required": True, "type": "str"}
                                        }},
                "dampening": {"required": False, "type": "str",
                              "choices": ["enable", "disable"]},
                "dampening-max-suppress-time": {"required": False, "type": "int"},
                "dampening-reachability-half-life": {"required": False, "type": "int"},
                "dampening-reuse": {"required": False, "type": "int"},
                "dampening-route-map": {"required": False, "type": "str"},
                "dampening-suppress": {"required": False, "type": "int"},
                "dampening-unreachability-half-life": {"required": False, "type": "int"},
                "default-local-preference": {"required": False, "type": "int"},
                "deterministic-med": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "distance-external": {"required": False, "type": "int"},
                "distance-internal": {"required": False, "type": "int"},
                "distance-local": {"required": False, "type": "int"},
                "ebgp-multipath": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "enforce-first-as": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "fast-external-failover": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "graceful-end-on-timer": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                "graceful-restart": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "graceful-restart-time": {"required": False, "type": "int"},
                "graceful-stalepath-time": {"required": False, "type": "int"},
                "graceful-update-delay": {"required": False, "type": "int"},
                "holdtime-timer": {"required": False, "type": "int"},
                "ibgp-multipath": {"required": False, "type": "str",
                                   "choices": ["enable", "disable"]},
                "ignore-optional-capability": {"required": False, "type": "str",
                                               "choices": ["enable", "disable"]},
                "keepalive-timer": {"required": False, "type": "int"},
                "log-neighbour-changes": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                "neighbor": {"required": False, "type": "list",
                             "options": {
                                 "activate": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                                 "activate6": {"required": False, "type": "str",
                                               "choices": ["enable", "disable"]},
                                 "advertisement-interval": {"required": False, "type": "int"},
                                 "allowas-in": {"required": False, "type": "int"},
                                 "allowas-in-enable": {"required": False, "type": "str",
                                                       "choices": ["enable", "disable"]},
                                 "allowas-in-enable6": {"required": False, "type": "str",
                                                        "choices": ["enable", "disable"]},
                                 "allowas-in6": {"required": False, "type": "int"},
                                 "as-override": {"required": False, "type": "str",
                                                 "choices": ["enable", "disable"]},
                                 "as-override6": {"required": False, "type": "str",
                                                  "choices": ["enable", "disable"]},
                                 "attribute-unchanged": {"required": False, "type": "str",
                                                         "choices": ["as-path", "med", "next-hop"]},
                                 "attribute-unchanged6": {"required": False, "type": "str",
                                                          "choices": ["as-path", "med", "next-hop"]},
                                 "bfd": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                                 "capability-default-originate": {"required": False, "type": "str",
                                                                  "choices": ["enable", "disable"]},
                                 "capability-default-originate6": {"required": False, "type": "str",
                                                                   "choices": ["enable", "disable"]},
                                 "capability-dynamic": {"required": False, "type": "str",
                                                        "choices": ["enable", "disable"]},
                                 "capability-graceful-restart": {"required": False, "type": "str",
                                                                 "choices": ["enable", "disable"]},
                                 "capability-graceful-restart6": {"required": False, "type": "str",
                                                                  "choices": ["enable", "disable"]},
                                 "capability-orf": {"required": False, "type": "str",
                                                    "choices": ["none", "receive", "send",
                                                                "both"]},
                                 "capability-orf6": {"required": False, "type": "str",
                                                     "choices": ["none", "receive", "send",
                                                                 "both"]},
                                 "capability-route-refresh": {"required": False, "type": "str",
                                                              "choices": ["enable", "disable"]},
                                 "conditional-advertise": {"required": False, "type": "list",
                                                           "options": {
                                                               "advertise-routemap": {"required": True, "type": "str"},
                                                               "condition-routemap": {"required": False, "type": "str"},
                                                               "condition-type": {"required": False, "type": "str",
                                                                                  "choices": ["exist", "non-exist"]}
                                                           }},
                                 "connect-timer": {"required": False, "type": "int"},
                                 "default-originate-routemap": {"required": False, "type": "str"},
                                 "default-originate-routemap6": {"required": False, "type": "str"},
                                 "description": {"required": False, "type": "str"},
                                 "distribute-list-in": {"required": False, "type": "str"},
                                 "distribute-list-in6": {"required": False, "type": "str"},
                                 "distribute-list-out": {"required": False, "type": "str"},
                                 "distribute-list-out6": {"required": False, "type": "str"},
                                 "dont-capability-negotiate": {"required": False, "type": "str",
                                                               "choices": ["enable", "disable"]},
                                 "ebgp-enforce-multihop": {"required": False, "type": "str",
                                                           "choices": ["enable", "disable"]},
                                 "ebgp-multihop-ttl": {"required": False, "type": "int"},
                                 "filter-list-in": {"required": False, "type": "str"},
                                 "filter-list-in6": {"required": False, "type": "str"},
                                 "filter-list-out": {"required": False, "type": "str"},
                                 "filter-list-out6": {"required": False, "type": "str"},
                                 "holdtime-timer": {"required": False, "type": "int"},
                                 "interface": {"required": False, "type": "str"},
                                 "ip": {"required": True, "type": "str"},
                                 "keep-alive-timer": {"required": False, "type": "int"},
                                 "link-down-failover": {"required": False, "type": "str",
                                                        "choices": ["enable", "disable"]},
                                 "local-as": {"required": False, "type": "int"},
                                 "local-as-no-prepend": {"required": False, "type": "str",
                                                         "choices": ["enable", "disable"]},
                                 "local-as-replace-as": {"required": False, "type": "str",
                                                         "choices": ["enable", "disable"]},
                                 "maximum-prefix": {"required": False, "type": "int"},
                                 "maximum-prefix-threshold": {"required": False, "type": "int"},
                                 "maximum-prefix-threshold6": {"required": False, "type": "int"},
                                 "maximum-prefix-warning-only": {"required": False, "type": "str",
                                                                 "choices": ["enable", "disable"]},
                                 "maximum-prefix-warning-only6": {"required": False, "type": "str",
                                                                  "choices": ["enable", "disable"]},
                                 "maximum-prefix6": {"required": False, "type": "int"},
                                 "next-hop-self": {"required": False, "type": "str",
                                                   "choices": ["enable", "disable"]},
                                 "next-hop-self6": {"required": False, "type": "str",
                                                    "choices": ["enable", "disable"]},
                                 "override-capability": {"required": False, "type": "str",
                                                         "choices": ["enable", "disable"]},
                                 "passive": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                                 "password": {"required": False, "type": "str"},
                                 "prefix-list-in": {"required": False, "type": "str"},
                                 "prefix-list-in6": {"required": False, "type": "str"},
                                 "prefix-list-out": {"required": False, "type": "str"},
                                 "prefix-list-out6": {"required": False, "type": "str"},
                                 "remote-as": {"required": False, "type": "int"},
                                 "remove-private-as": {"required": False, "type": "str",
                                                       "choices": ["enable", "disable"]},
                                 "remove-private-as6": {"required": False, "type": "str",
                                                        "choices": ["enable", "disable"]},
                                 "restart-time": {"required": False, "type": "int"},
                                 "retain-stale-time": {"required": False, "type": "int"},
                                 "route-map-in": {"required": False, "type": "str"},
                                 "route-map-in6": {"required": False, "type": "str"},
                                 "route-map-out": {"required": False, "type": "str"},
                                 "route-map-out6": {"required": False, "type": "str"},
                                 "route-reflector-client": {"required": False, "type": "str",
                                                            "choices": ["enable", "disable"]},
                                 "route-reflector-client6": {"required": False, "type": "str",
                                                             "choices": ["enable", "disable"]},
                                 "route-server-client": {"required": False, "type": "str",
                                                         "choices": ["enable", "disable"]},
                                 "route-server-client6": {"required": False, "type": "str",
                                                          "choices": ["enable", "disable"]},
                                 "send-community": {"required": False, "type": "str",
                                                    "choices": ["standard", "extended", "both",
                                                                "disable"]},
                                 "send-community6": {"required": False, "type": "str",
                                                     "choices": ["standard", "extended", "both",
                                                                 "disable"]},
                                 "shutdown": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                                 "soft-reconfiguration": {"required": False, "type": "str",
                                                          "choices": ["enable", "disable"]},
                                 "soft-reconfiguration6": {"required": False, "type": "str",
                                                           "choices": ["enable", "disable"]},
                                 "stale-route": {"required": False, "type": "str",
                                                 "choices": ["enable", "disable"]},
                                 "strict-capability-match": {"required": False, "type": "str",
                                                             "choices": ["enable", "disable"]},
                                 "unsuppress-map": {"required": False, "type": "str"},
                                 "unsuppress-map6": {"required": False, "type": "str"},
                                 "update-source": {"required": False, "type": "str"},
                                 "weight": {"required": False, "type": "int"}
                             }},
                "neighbor-group": {"required": False, "type": "list",
                                   "options": {
                                       "activate": {"required": False, "type": "str",
                                                    "choices": ["enable", "disable"]},
                                       "activate6": {"required": False, "type": "str",
                                                     "choices": ["enable", "disable"]},
                                       "advertisement-interval": {"required": False, "type": "int"},
                                       "allowas-in": {"required": False, "type": "int"},
                                       "allowas-in-enable": {"required": False, "type": "str",
                                                             "choices": ["enable", "disable"]},
                                       "allowas-in-enable6": {"required": False, "type": "str",
                                                              "choices": ["enable", "disable"]},
                                       "allowas-in6": {"required": False, "type": "int"},
                                       "as-override": {"required": False, "type": "str",
                                                       "choices": ["enable", "disable"]},
                                       "as-override6": {"required": False, "type": "str",
                                                        "choices": ["enable", "disable"]},
                                       "attribute-unchanged": {"required": False, "type": "str",
                                                               "choices": ["as-path", "med", "next-hop"]},
                                       "attribute-unchanged6": {"required": False, "type": "str",
                                                                "choices": ["as-path", "med", "next-hop"]},
                                       "bfd": {"required": False, "type": "str",
                                               "choices": ["enable", "disable"]},
                                       "capability-default-originate": {"required": False, "type": "str",
                                                                        "choices": ["enable", "disable"]},
                                       "capability-default-originate6": {"required": False, "type": "str",
                                                                         "choices": ["enable", "disable"]},
                                       "capability-dynamic": {"required": False, "type": "str",
                                                              "choices": ["enable", "disable"]},
                                       "capability-graceful-restart": {"required": False, "type": "str",
                                                                       "choices": ["enable", "disable"]},
                                       "capability-graceful-restart6": {"required": False, "type": "str",
                                                                        "choices": ["enable", "disable"]},
                                       "capability-orf": {"required": False, "type": "str",
                                                          "choices": ["none", "receive", "send",
                                                                      "both"]},
                                       "capability-orf6": {"required": False, "type": "str",
                                                           "choices": ["none", "receive", "send",
                                                                       "both"]},
                                       "capability-route-refresh": {"required": False, "type": "str",
                                                                    "choices": ["enable", "disable"]},
                                       "connect-timer": {"required": False, "type": "int"},
                                       "default-originate-routemap": {"required": False, "type": "str"},
                                       "default-originate-routemap6": {"required": False, "type": "str"},
                                       "description": {"required": False, "type": "str"},
                                       "distribute-list-in": {"required": False, "type": "str"},
                                       "distribute-list-in6": {"required": False, "type": "str"},
                                       "distribute-list-out": {"required": False, "type": "str"},
                                       "distribute-list-out6": {"required": False, "type": "str"},
                                       "dont-capability-negotiate": {"required": False, "type": "str",
                                                                     "choices": ["enable", "disable"]},
                                       "ebgp-enforce-multihop": {"required": False, "type": "str",
                                                                 "choices": ["enable", "disable"]},
                                       "ebgp-multihop-ttl": {"required": False, "type": "int"},
                                       "filter-list-in": {"required": False, "type": "str"},
                                       "filter-list-in6": {"required": False, "type": "str"},
                                       "filter-list-out": {"required": False, "type": "str"},
                                       "filter-list-out6": {"required": False, "type": "str"},
                                       "holdtime-timer": {"required": False, "type": "int"},
                                       "interface": {"required": False, "type": "str"},
                                       "keep-alive-timer": {"required": False, "type": "int"},
                                       "link-down-failover": {"required": False, "type": "str",
                                                              "choices": ["enable", "disable"]},
                                       "local-as": {"required": False, "type": "int"},
                                       "local-as-no-prepend": {"required": False, "type": "str",
                                                               "choices": ["enable", "disable"]},
                                       "local-as-replace-as": {"required": False, "type": "str",
                                                               "choices": ["enable", "disable"]},
                                       "maximum-prefix": {"required": False, "type": "int"},
                                       "maximum-prefix-threshold": {"required": False, "type": "int"},
                                       "maximum-prefix-threshold6": {"required": False, "type": "int"},
                                       "maximum-prefix-warning-only": {"required": False, "type": "str",
                                                                       "choices": ["enable", "disable"]},
                                       "maximum-prefix-warning-only6": {"required": False, "type": "str",
                                                                        "choices": ["enable", "disable"]},
                                       "maximum-prefix6": {"required": False, "type": "int"},
                                       "name": {"required": True, "type": "str"},
                                       "next-hop-self": {"required": False, "type": "str",
                                                         "choices": ["enable", "disable"]},
                                       "next-hop-self6": {"required": False, "type": "str",
                                                          "choices": ["enable", "disable"]},
                                       "override-capability": {"required": False, "type": "str",
                                                               "choices": ["enable", "disable"]},
                                       "passive": {"required": False, "type": "str",
                                                   "choices": ["enable", "disable"]},
                                       "prefix-list-in": {"required": False, "type": "str"},
                                       "prefix-list-in6": {"required": False, "type": "str"},
                                       "prefix-list-out": {"required": False, "type": "str"},
                                       "prefix-list-out6": {"required": False, "type": "str"},
                                       "remote-as": {"required": False, "type": "int"},
                                       "remove-private-as": {"required": False, "type": "str",
                                                             "choices": ["enable", "disable"]},
                                       "remove-private-as6": {"required": False, "type": "str",
                                                              "choices": ["enable", "disable"]},
                                       "restart-time": {"required": False, "type": "int"},
                                       "retain-stale-time": {"required": False, "type": "int"},
                                       "route-map-in": {"required": False, "type": "str"},
                                       "route-map-in6": {"required": False, "type": "str"},
                                       "route-map-out": {"required": False, "type": "str"},
                                       "route-map-out6": {"required": False, "type": "str"},
                                       "route-reflector-client": {"required": False, "type": "str",
                                                                  "choices": ["enable", "disable"]},
                                       "route-reflector-client6": {"required": False, "type": "str",
                                                                   "choices": ["enable", "disable"]},
                                       "route-server-client": {"required": False, "type": "str",
                                                               "choices": ["enable", "disable"]},
                                       "route-server-client6": {"required": False, "type": "str",
                                                                "choices": ["enable", "disable"]},
                                       "send-community": {"required": False, "type": "str",
                                                          "choices": ["standard", "extended", "both",
                                                                      "disable"]},
                                       "send-community6": {"required": False, "type": "str",
                                                           "choices": ["standard", "extended", "both",
                                                                       "disable"]},
                                       "shutdown": {"required": False, "type": "str",
                                                    "choices": ["enable", "disable"]},
                                       "soft-reconfiguration": {"required": False, "type": "str",
                                                                "choices": ["enable", "disable"]},
                                       "soft-reconfiguration6": {"required": False, "type": "str",
                                                                 "choices": ["enable", "disable"]},
                                       "stale-route": {"required": False, "type": "str",
                                                       "choices": ["enable", "disable"]},
                                       "strict-capability-match": {"required": False, "type": "str",
                                                                   "choices": ["enable", "disable"]},
                                       "unsuppress-map": {"required": False, "type": "str"},
                                       "unsuppress-map6": {"required": False, "type": "str"},
                                       "update-source": {"required": False, "type": "str"},
                                       "weight": {"required": False, "type": "int"}
                                   }},
                "neighbor-range": {"required": False, "type": "list",
                                   "options": {
                                       "id": {"required": True, "type": "int"},
                                       "max-neighbor-num": {"required": False, "type": "int"},
                                       "neighbor-group": {"required": False, "type": "str"},
                                       "prefix": {"required": False, "type": "str"}
                                   }},
                "network": {"required": False, "type": "list",
                            "options": {
                                "backdoor": {"required": False, "type": "str",
                                             "choices": ["enable", "disable"]},
                                "id": {"required": True, "type": "int"},
                                "prefix": {"required": False, "type": "str"},
                                "route-map": {"required": False, "type": "str"}
                            }},
                "network-import-check": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "network6": {"required": False, "type": "list",
                             "options": {
                                 "backdoor": {"required": False, "type": "str",
                                              "choices": ["enable", "disable"]},
                                 "id": {"required": True, "type": "int"},
                                 "prefix6": {"required": False, "type": "str"},
                                 "route-map": {"required": False, "type": "str"}
                             }},
                "redistribute": {"required": False, "type": "list",
                                 "options": {
                                     "name": {"required": True, "type": "str"},
                                     "route-map": {"required": False, "type": "str"},
                                     "status": {"required": False, "type": "str",
                                                "choices": ["enable", "disable"]}
                                 }},
                "redistribute6": {"required": False, "type": "list",
                                  "options": {
                                      "name": {"required": True, "type": "str"},
                                      "route-map": {"required": False, "type": "str"},
                                      "status": {"required": False, "type": "str",
                                                 "choices": ["enable", "disable"]}
                                  }},
                "router-id": {"required": False, "type": "str"},
                "scan-time": {"required": False, "type": "int"},
                "synchronization": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_router(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



"""
iap_token unit tests
"""
# -*- coding: utf-8 -*-

# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# pylint: disable=invalid-name,protected-access,function-redefined,unused-argument
# pylint: disable=unused-import,redundant-unittest-assert

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


import unittest


class TestClass(unittest.TestCase):
    """
    Test cases
    """
    def _assert_incident_api(self, module, url, method, headers):
        """
        Setup Test
        """
        self.assertTrue('http://localhost:4007/login' in url, 'token')
        return Response(), {'status': 200}

    def test_incident_url(self):
        self.assertTrue(True, True)


class Response(object):
    """
    Setup Response
    """
    def read(self):
        return '{"token": "ljhklj%3D"}'



import json
import pytest

from ansible.module_utils import kubevirt as mymodule


def test_simple_merge_dicts():
    dict1 = {'labels': {'label1': 'value'}}
    dict2 = {'labels': {'label2': 'value'}}
    dict3 = json.dumps({'labels': {'label1': 'value', 'label2': 'value'}}, sort_keys=True)
    assert dict3 == json.dumps(dict(mymodule.KubeVirtRawModule.merge_dicts(dict1, dict2)), sort_keys=True)


def test_simple_multi_merge_dicts():
    dict1 = {'labels': {'label1': 'value', 'label3': 'value'}}
    dict2 = {'labels': {'label2': 'value'}}
    dict3 = json.dumps({'labels': {'label1': 'value', 'label2': 'value', 'label3': 'value'}}, sort_keys=True)
    assert dict3 == json.dumps(dict(mymodule.KubeVirtRawModule.merge_dicts(dict1, dict2)), sort_keys=True)


def test_double_nested_merge_dicts():
    dict1 = {'metadata': {'labels': {'label1': 'value', 'label3': 'value'}}}
    dict2 = {'metadata': {'labels': {'label2': 'value'}}}
    dict3 = json.dumps({'metadata': {'labels': {'label1': 'value', 'label2': 'value', 'label3': 'value'}}}, sort_keys=True)
    assert dict3 == json.dumps(dict(mymodule.KubeVirtRawModule.merge_dicts(dict1, dict2)), sort_keys=True)
